from trame_common.decorators.hot_reload import hot_reload
from trame_common.decorators.klass import (
    TrameApp,
    change,
    controller,
    life_cycle,
    trigger,
)
from trame_common.exec.asynchronous import task

__all__ = [
    "TrameApp",
    "change",
    "trigger",
    "controller",
    "life_cycle",
    "hot_reload",
    "task",
]



from tkinter import Tk, filedialog

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# Keep track of the currently selected directory
state.selected_dir = "None"

root = Tk()

# Ensure the tkinter main window is hidden
root.withdraw()

# Ensure that the file browser will appear in front on Windows
root.wm_attributes("-topmost", 1)


def open_directory():
    kwargs = {
        "title": "Select Directory",
    }
    dirpath = filedialog.askdirectory(**kwargs)
    if not dirpath:
        # User canceled.
        print("Canceled")
        return

    print("Selected directory:", dirpath)
    state.selected_dir = dirpath


# The controller would let us set up callback logic in a separate file
ctrl.open_directory = open_directory

# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

layout = SinglePageLayout(server)

with layout:
    # Toolbar
    with layout.toolbar as toolbar:
        toolbar.clear()

        vuetify.VSpacer()

        vuetify.VBtn("Select Directory", click=ctrl.open_directory)

        vuetify.VSpacer()

        with layout.content:
            with vuetify.VContainer() as container:
                container.add_child("Selected Directory: {{ selected_dir }}")

# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from pathlib import Path

import vtkmodules.vtkRenderingOpenGL2  # noqa

# VTK factory initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridReader
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkDataSetMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame import controller as ctrl
from trame.html import vtk, vuetify
from trame.layouts import SinglePage

DATA_DIR = Path(Path(__file__).parent.parent.parent.parent, "data")
MESH_PATH = str(Path(DATA_DIR, "mesh.vtu"))

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

reader = vtkXMLUnstructuredGridReader()
reader.SetFileName(MESH_PATH)

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

mapper = vtkDataSetMapper()
actor = vtkActor()
actor.GetProperty().SetEdgeVisibility(1)
mapper.SetInputConnection(reader.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

layout = SinglePage("VTK.js rendering", on_ready=ctrl.on_ready)
layout.title.set_text("Test mesh exchange")

with layout.content:
    with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
        with vtk.VtkRemoteView(renderWindow) as view:
            layout.logo.click = view.reset_camera
            ctrl.on_ready = view.update

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    layout.start()



from trame.app import get_server
from trame.decorators import TrameApp, change, controller, life_cycle, trigger
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify


@TrameApp()
class App:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue2")
        self.ui()

    @property
    def ctrl(self):
        return self.server.controller

    @trigger("exec")
    def method_call(self, *args):
        print("method_called", args)

    @controller.set("hello")
    def method_on_ctrl(self, *args):
        print("method_on_ctrl", args)

    @change("slider_value1", "slider_value2")
    def both_sliders(self, slider_value1, slider_value2, **kwargs):
        print("Slider value (1 or 2)", slider_value1, slider_value2)

    @change("slider_value1")
    def one_slider(self, slider_value1, **kwargs):
        print("Slider value 1", slider_value1)

    @life_cycle.server_ready
    def on_ready(self, *args, **kwargs):
        print("on_ready")

    def ui(self):
        with SinglePageLayout(self.server) as layout:
            with layout.toolbar:
                vuetify.VSpacer()
                vuetify.VSlider(
                    v_model=("slider_value1", 0), dense=True, hide_details=True
                )
                vuetify.VSlider(
                    v_model=("slider_value2", 0), dense=True, hide_details=True
                )
                vuetify.VBtn("trigger", click="trigger('exec')")
                vuetify.VBtn("method", click=self.method_call)
                vuetify.VBtn("ctrl", click=self.ctrl.hello)


app = App()
app.server.start()



from trame.app import get_server
from trame.ui.html import DivLayout
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import client, html, trame, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

LAYOUTS = {}
state.last_line_count = 1

# -----------------------------------------------------------------------------
# Actions
# -----------------------------------------------------------------------------


def add_line(layout_name):
    layout = LAYOUTS[layout_name]
    with layout:
        with layout.content:
            html.Div(f"Line - {state.last_line_count}")
            state.last_line_count += 1


# -----------------------------------------------------------------------------
# Layouts
# -----------------------------------------------------------------------------

for name in "abcdef":
    layout = DivLayout(server, name)
    LAYOUTS[name] = layout
    with layout:
        trame.LifeCycleMonitor(name=f"{name} Layout", events=("['created']",))
        with vuetify.VCard():
            with vuetify.VCardTitle(f"Layout {name}") as title:
                vuetify.VSpacer()
                title.add_child("{{ slider_value }}")
                vuetify.VSpacer()
                with vuetify.VBtn(icon=True, click=(add_line, f"['{name}']")):
                    vuetify.VIcon("mdi-plus")
            vuetify.VDivider()
            with vuetify.VCardText():
                layout.content = vuetify.VCol()

# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

main_layout = SinglePageLayout(server)

with main_layout:
    # Validate client life cycle
    trame.LifeCycleMonitor(name="Main Layout", events=("['created']",))

    main_layout.title.set_text("Many Layouts")
    main_layout.toolbar.dense = True

    # Toolbar
    with main_layout.toolbar as toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("slider_value", 5),
            min=1,
            max=10,
            dense=True,
            hide_details=True,
            style="max-width: 300px;",
        )
        toolbar.add_child("{{ last_line_count }}")

    # Multi-layout
    with main_layout.content:
        with vuetify.VContainer(fluid=True):
            with vuetify.VRow():
                for key in LAYOUTS:
                    with vuetify.VCol(cols=2):
                        client.ServerTemplate(name=key, classes="pa-4")

# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.decorators import hot_reload
from trame.ui.html import DivLayout
from trame.widgets import html

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server()
state, ctrl = server.state, server.controller

# Can be enabled at runtime as well with
# 1. export TRAME_HOT_RELOAD=1
# 2. --hot-reload
server.hot_reload = True


# -----------------------------------------------------------------------------
# Dynamically modify any `ChangeMe` to see the new code execute while
# interacting with the app.
# -----------------------------------------------------------------------------
@ctrl.set("number_reset")
def reset_number():
    print("reset_number::ChangeMe 2")
    state.number = 6
    state.size = 1
    do_someting()


@state.change("number")
def update_number(number, **kwargs):
    print("update_number::ChangeMe", number)
    do_someting()


@hot_reload
def do_someting():
    print("do_someting::ChangeMe")


@ctrl.set("update_ui")
def update_ui():
    with DivLayout(server):
        html.Div("Some content - ChangeMe")
        html.Input(type="range", v_model_number=("number", 6))
        html.Input(type="range", v_model_number=("size", 2))
        html.Button("Reset", click=ctrl.number_reset)
        html.Button("Update", click=ctrl.update_ui)
        html.Div(
            "{{ number }} x {{ i }} = {{ number * i }}",
            v_for="i in size",
            key="i",
        )


# Need to run before start
update_ui()

# -----------------------------------------------------------------------------
# Automatic UI update on file change using watchdog (pip install watchdog)
# -----------------------------------------------------------------------------

try:
    import asyncio
    from pathlib import Path

    from watchdog.events import FileSystemEventHandler
    from watchdog.observers import Observer

    current_event_loop = asyncio.get_event_loop()

    def update_ui():
        with server.state:
            ctrl.update_ui()

    class UpdateUIOnChange(FileSystemEventHandler):
        def on_modified(self, event):
            current_event_loop.call_soon_threadsafe(update_ui)

    observer = Observer()
    observer.schedule(
        UpdateUIOnChange(), str(Path(__file__).parent.absolute()), recursive=False
    )
    observer.start()
except ImportError:
    print("Watchdog not installed so skipping the auto monitoring")

# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, trame, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state = server.state

CURSORS = [
    "auto",
    "default",
    "none",
    "context-menu",
    "help",
    "pointer",
    "progress",
    "wait",
    "cell",
    "crosshair",
    "text",
    "vertical-text",
    "alias",
    "copy",
    "move",
    "no-drop",
    "not-allowed",
    "grab",
    "grabbing",
    "e-resize",
    "n-resize",
    "ne-resize",
    "nw-resize",
    "s-resize",
    "se-resize",
    "sw-resize",
    "w-resize",
    "ew-resize",
    "ns-resize",
    "nesw-resize",
    "nwse-resize",
    "col-resize",
    "row-resize",
    "all-scroll",
    "zoom-in",
    "zoom-out",
]

# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = f"active = (active + 1) % {len(CURSORS)}"
    layout.title.set_text("Toggle Cursor")

    trame.Cursor(
        active=("active", 0),
        cursors=("options", CURSORS),
    )

    with layout.toolbar:
        trame.Cursor(
            active=("active + 1",),
            cursors=("options", CURSORS),
        )
        html.Div(
            "({{ options[active + 1] }})",
            classes="ml-2",
            style="text-overflow: ellipsis;",
        )
        vuetify.VSpacer()
        vuetify.VChip("{{ active }}")
        vuetify.VSlider(
            dense=True,
            hide_details=True,
            v_model="active",
            min=0,
            max=len(CURSORS) - 1,
            style="max-width: 200px;",
        )

    with layout.content:
        with vuetify.VContainer(fluid=True):
            with vuetify.VCol():
                with vuetify.VRow(style="background: red;"):
                    html.Div("{{ options[active + 2] }}", classes="pa-6")
                    trame.Cursor(
                        active=("active + 2",),
                        cursors=("options", CURSORS),
                    )
                with vuetify.VRow():
                    html.Div("{{ options[active] }}", classes="pa-6")


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state = server.state

state.chips = [
    dict(name="hello", id=1, show=True),
    dict(name="world", id=2, show=True),
    dict(name="seb", id=3, show=True),
]


@state.change("chip_group")
def on_active(chip_group, **kwargs):
    print("chip_group", chip_group)


def remove(chip_id):
    print("remove", chip_id)
    for item in state.chips:
        if item.get("id") == chip_id:
            item["show"] = False
    state.dirty("chips")


# -----------------------------------------------------------------------------
# GUI layout
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    with layout.content:
        with vuetify.VContainer(fluid=True, classes="fill-height pa-0 ma-0"):
            with vuetify.VChipGroup(
                v_model=("chip_group", None), active_class="primary", mandatory=True
            ):
                vuetify.VChip("Dashboard", value="dashboard", label=True, outlined=True)
                vuetify.VChip(
                    "{{ item.name }}",
                    v_for="item, idx in chips",
                    key="idx",
                    value=("item.id",),
                    v_show="item.show",
                    close=True,
                    click_close=(remove, "[item.id]"),
                    label=True,
                    outlined=True,
                )

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



# pip install trame trame-vtk trame-vuetify

from typing import Any

import paraview.web.venv
from paraview import simple
from trame_server import Server
from trame_server.controller import Controller

from trame.app import get_server
from trame.decorators import TrameApp, change
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import client, paraview
from trame.widgets import vuetify3 as vuetify


@TrameApp()
class Viewer:
    def __init__(self, server: Server = None) -> None:
        self.server = get_server(server, client_type="vue3")

        self.pixel_ratio = 1

        self.load_cone_source()
        self.setup_ui()

    @property
    def ctrl(self) -> Controller:
        return self.server.controller

    @property
    def active_view(self) -> Any:
        return simple.GetActiveViewOrCreate("RenderView")

    @property
    def active_camera(self) -> Any:
        return simple.GetActiveCamera()

    def load_cone_source(self) -> None:
        cone = simple.Cone()
        simple.Show(cone)
        simple.Render()

    def add_sphere(self, sphere_center: list, color: list) -> None:
        sphere = simple.Sphere()
        sphere.Center = sphere_center
        sphere.Radius = 0.01
        sphere_display = simple.Show(sphere)
        sphere_display.AmbientColor = color
        sphere_display.DiffuseColor = color

        self.ctrl.view_update()

    def on_right_button_release(self, pickData: dict) -> None:
        mouse_position = pickData["position"]
        world_position = [0, 0, 0]
        normal = [0, 0, 0]
        on_mesh = self.active_view.ConvertDisplayToPointOnSurface(
            [
                int(mouse_position["x"] / self.pixel_ratio),
                int(mouse_position["y"] / self.pixel_ratio),
            ],
            world_position,
            normal,
        )

        sphere_color = [0.0, 1.0, 0.0] if on_mesh else [1.0, 0.0, 0.0]
        self.add_sphere(world_position, sphere_color)

    def on_end_animation(self, camera_info: dict) -> None:
        # Synchronize cameras
        self.active_camera.SetPosition(camera_info.get("position"))
        self.active_camera.SetFocalPoint(camera_info.get("focalPoint"))
        self.active_camera.SetViewUp(camera_info.get("viewUp"))
        self.active_camera.SetViewAngle(camera_info.get("viewAngle"))

    @change("view_size")
    def on_view_size_change(self, view_size: dict, **kwargs):
        # Synchronize views' size
        if view_size:
            self.pixel_ratio = view_size["pixelRatio"]
            self.active_view.ViewSize = [
                int(view_size["size"]["width"]),
                int(view_size["size"]["height"]),
            ]

    def setup_ui(self) -> None:
        with SinglePageLayout(self.server) as layout:
            with (
                layout.content,
                vuetify.VContainer(fluid=True, classes="ma-0 pa-0 grey fill-height"),
                client.SizeObserver("view_size"),
            ):
                view = paraview.VtkLocalView(
                    self.active_view,
                    ref="view",
                    interactor_events=(
                        "events",
                        ["RightButtonRelease", "EndAnimation"],
                    ),
                    RightButtonRelease=(
                        self.on_right_button_release,
                        "[utils.vtk.event($event)]",
                    ),
                    EndAnimation=(
                        self.on_end_animation,
                        "[$event.pokedRenderer.getActiveCamera().get()]",
                    ),
                )
                self.ctrl.view_update = view.update


if __name__ == "__main__":
    app = Viewer()
    app.server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/ContourGeometry/ClientView.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/3852cba56cd63f6efa684d5e5fb00881a6111e81

Installation requirements:
    pip install trame trame-vuetify trame-vtk vtk
"""

from pathlib import Path

from vtkmodules.vtkFiltersCore import vtkContourFilter
from vtkmodules.vtkIOXML import vtkXMLImageDataReader

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "VTK contour - Remote/Local rendering"

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

data_directory = Path(__file__).parent.parent.with_name("data")
head_vti = data_directory / "head.vti"

reader = vtkXMLImageDataReader()
reader.SetFileName(head_vti)
reader.Update()

contour = vtkContourFilter()
contour.SetInputConnection(reader.GetOutputPort())
contour.SetComputeNormals(1)
contour.SetComputeScalars(0)

# Extract data range => Update store/state
data_range = reader.GetOutput().GetPointData().GetScalars().GetRange()
contour_value = 0.5 * (data_range[0] + data_range[1])

# Configure contour with valid values
contour.SetNumberOfContours(1)
contour.SetValue(0, contour_value)

# Share with client
state.data_range = data_range

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("contour_value")
def update_contour(contour_value, **kwargs):
    contour.SetValue(0, contour_value)
    ctrl.ds_update()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.title.set_text("Contour Application - Local rendering")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            value=("contour_value", contour_value),
            min=("data_range[0]",),
            max=("data_range[1]",),
            hide_details=True,
            dense=True,
            style="max-width: 300px",
            change="contour_value = Number($event)",
        )
        vuetify.VSwitch(
            v_model="$vuetify.theme.dark",
            hide_details=True,
        )

        with vuetify.VBtn(icon=True, click="$refs.view.resetCamera()"):
            vuetify.VIcon("mdi-crop-free")

        vuetify.VProgressLinear(
            indeterminate=True,
            absolute=True,
            bottom=True,
            active=("trame__busy",),
        )

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            with vtk_widgets.VtkView() as view:
                layout.icon.click = view.reset_camera
                with vtk_widgets.VtkGeometryRepresentation():
                    polydata = vtk_widgets.VtkPolyData("contour", dataset=contour)
                    ctrl.ds_update = polydata.update

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



#!/usr/bin/env python3

import json
import sys

import yaml

if len(sys.argv) < 3:
    sys.exit("Usage: <script> <input_yaml> <output_json>")

input_file = sys.argv[1]
output_file = sys.argv[2]

with open(input_file, "r") as rf:
    input_dict = yaml.safe_load(rf)

with open(output_file, "w") as wf:
    json.dump(input_dict, wf, indent=2)



#

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html


class ExternalJS:
    def __init__(self, server=None):
        self.server = get_server(server)

        self.server.enable_module(
            {
                "module_scripts": [
                    "https://esm.sh/canvas-confetti@1.9.3",
                ],
            }
        )
        self.ui = self._build_ui()

    def _build_ui(self):
        with DivLayout(self.server):
            html.Button("Click Me")


def main():
    app = ExternalJS()
    app.server.start()


if __name__ == "__main__":
    main()



# To run with Paraview
# pvpython ./examples/validation/core/23_pv_interactive_render_exit.py --venv .venv
import paraview.web.venv

# -----------------------------------------------------------------------------
# VTK
# -----------------------------------------------------------------------------
from paraview import simple

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html, paraview

DEFAULT_RESOLUTION = 8

cone = simple.Cone()
sphere = simple.Sphere()

simple.Show(cone)
simple.Show(sphere)

view = simple.Render()

# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

server = get_server()
state, ctrl = server.state, server.controller

with DivLayout(server) as layout:
    container = layout.root
    container.style = "width: 100vw; height: 100vh;"
    html_view = paraview.VtkRemoteView(view, interactive_ratio=1, interactive_quality=0)
    ctrl.view_update = html_view.update
    html.Input(
        type="range",
        min=8,
        max=1000,
        step=1,
        v_model=("resolution", 8),
        style="position: absolute; top: 20px; left: 20px; z-index: 1; width: 25%; min-width: 300px;",
    )


@state.change("resolution")
def update_resolution(resolution, **kwargs):
    cone.Resolution = int(resolution)
    sphere.PhiResolution = int(resolution)
    sphere.ThetaResolution = int(resolution)
    ctrl.view_update()


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import time

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html

server = get_server()
state, ctrl = server.state, server.controller

state.other_txt_content = "Some content to download..."


@ctrl.trigger("download_content")
def generate_content():
    return f"Hello on the server is {time.time()}"


with DivLayout(server):
    html.Button(
        "Download from method",
        click="utils.download('method.txt', trigger('download_content'), 'text/plain')",
    )
    html.Button(
        "Download from state",
        click="utils.download('state.txt', other_txt_content, 'text/plain')",
    )

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-components trame-matplotlib
"""

import matplotlib.pyplot as plt
import numpy as np

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import matplotlib, trame, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Chart examples from:
#   - http://jakevdp.github.io/blog/2013/12/19/a-d3-viewer-for-matplotlib/
# -----------------------------------------------------------------------------


def figure_size():
    if state.figure_size is None:
        return {}

    dpi = state.figure_size.get("dpi")
    rect = state.figure_size.get("size")
    w_inch = rect.get("width") / dpi
    h_inch = rect.get("height") / dpi

    return {
        "figsize": (w_inch, h_inch),
        "dpi": dpi,
    }


def FirstDemo():
    plt.close("all")
    fig, ax = plt.subplots(**figure_size())
    np.random.seed(0)
    ax.plot(
        np.random.normal(size=100), np.random.normal(size=100), "or", ms=10, alpha=0.3
    )
    ax.plot(
        np.random.normal(size=100), np.random.normal(size=100), "ob", ms=20, alpha=0.1
    )

    ax.set_xlabel("this is x")
    ax.set_ylabel("this is y")
    ax.set_title("Matplotlib Plot Rendered in D3!", size=14)
    ax.grid(color="lightgray", alpha=0.7)

    return fig


# -----------------------------------------------------------------------------


def MultiLines():
    plt.close("all")
    fig, ax = plt.subplots(**figure_size())
    x = np.linspace(0, 10, 1000)
    for offset in np.linspace(0, 3, 7):
        ax.plot(x, 0.9 * np.sin(x - offset), lw=5, alpha=0.4)
    ax.set_ylim(-1.2, 1.0)
    ax.text(5, -1.1, "Here are some curves", size=18)
    ax.grid(color="lightgray", alpha=0.7)

    return fig


# -----------------------------------------------------------------------------


def DotsandPoints():
    plt.close("all")
    fig, ax = plt.subplots(**figure_size())
    ax.plot(
        np.random.rand(20),
        "-o",
        alpha=0.5,
        color="black",
        linewidth=5,
        markerfacecolor="green",
        markeredgecolor="lightgreen",
        markersize=20,
        markeredgewidth=10,
    )
    ax.grid(True, color="#EEEEEE", linestyle="solid")
    ax.set_xlim(-2, 22)
    ax.set_ylim(-0.1, 1.1)

    return fig


# -----------------------------------------------------------------------------


def MovingWindowAverage():
    np.random.seed(0)
    t = np.linspace(0, 10, 300)
    x = np.sin(t)
    dx = np.random.normal(0, 0.3, 300)

    kernel = np.ones(25) / 25.0
    x_smooth = np.convolve(x + dx, kernel, mode="same")
    plt.close("all")
    fig, ax = plt.subplots(**figure_size())
    ax.plot(t, x + dx, linestyle="", marker="o", color="black", markersize=3, alpha=0.3)
    ax.plot(t, x_smooth, "-k", lw=3)
    ax.plot(t, x, "--", lw=3, color="blue")

    return fig


# -----------------------------------------------------------------------------


def Subplots():
    plt.close("all")
    fig = plt.figure(**figure_size())
    fig.subplots_adjust(hspace=0.3)

    np.random.seed(0)

    for i in range(1, 5):
        ax = fig.add_subplot(2, 2, i)
        color = np.random.random(3)
        ax.plot(np.random.random(30), lw=2, c=color)
        ax.set_title("RGB = ({0:.2f}, {1:.2f}, {2:.2f})".format(*color), size=14)
        ax.grid(color="lightgray", alpha=0.7)

    return fig


# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("active_figure", "figure_size")
def update_chart(active_figure, **kwargs):
    ctrl.update_figure(globals()[active_figure]())


# -----------------------------------------------------------------------------
# UI
# -----------------------------------------------------------------------------

state.trame__title = "Matplotly"

with SinglePageLayout(server) as layout:
    layout.title.set_text("trame ❤️ matplotlib")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSelect(
            v_model=("active_figure", "FirstDemo"),
            items=(
                "figures",
                [
                    {"text": "First Demo", "value": "FirstDemo"},
                    {"text": "Multi Lines", "value": "MultiLines"},
                    {"text": "Dots and Points", "value": "DotsandPoints"},
                    {"text": "Moving Window Average", "value": "MovingWindowAverage"},
                    {"text": "Subplots", "value": "Subplots"},
                ],
            ),
            hide_details=True,
            dense=True,
        )

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="fill-height pa-0 ma-0"):
            with trame.SizeObserver("figure_size"):
                html_figure = matplotlib.Figure(style="position: absolute")
                ctrl.update_figure = html_figure.update

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk vtk
"""

import tempfile
from urllib.error import HTTPError
from urllib.request import urlretrieve

import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersGeometry import vtkGeometryFilter

# VTK imports
from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridReader
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkDataSetMapper,
    vtkRenderer,
    vtkRenderWindow,
)

# Trame imports
from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify3 as vuetify

server = get_server(client_type="vue3")
server.cli.add_argument(
    "--remote-url",
    dest="remote_url",
    help="Remote URL pointing to a VTU file.",
    default="https://github.com/Kitware/trame/raw/master/examples/data/disk_out_ref.vtu",
)
(args, _unknown) = server.cli.parse_known_args()

state, ctrl = server.state, server.controller

with tempfile.NamedTemporaryFile() as fp:
    try:
        print(f"using local file {fp.name}")
        urlretrieve(args.remote_url, fp.name)
        reader = vtkXMLUnstructuredGridReader()
        reader.SetFileName(fp.name)
        reader.Update()
    except HTTPError as e:
        print(RuntimeError(f"Failed to download {args.remote_url}. {e.reason}"))
        raise e


dataset = reader.GetOutput()

renderer = vtkRenderer()
renderer.SetBackground(1, 1, 1)
render_window = vtkRenderWindow()
render_window.AddRenderer(renderer)

surface_filter = vtkGeometryFilter()
surface_filter.SetInputConnection(reader.GetOutputPort())
surface_filter.SetPassThroughPointIds(True)

mapper = vtkDataSetMapper()
mapper.SetInputConnection(surface_filter.GetOutputPort())
actor = vtkActor()
actor.GetProperty().SetOpacity(0.5)
actor.SetMapper(mapper)

renderer.AddActor(actor)
renderer.ResetCamera()


state.trame__title = "Fetch Remote URL example"
ctrl.on_server_ready.add(ctrl.view_update)

with SinglePageLayout(server) as layout:
    layout.title.set_text("Unstructured grid from remote URL")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="fill-height pa-0 ma-0"):
            with vuetify.VRow(dense=True, style="height: 100%;"):
                with vuetify.VCol(
                    classes="pa-0",
                    style="border-right: 1px solid #ccc; position: relative;",
                ):
                    view = vtk_widgets.VtkLocalView(
                        render_window,
                    )
                    ctrl.view_update = view.update

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vega altair pandas numpy
"""

from itertools import cycle

import altair as alt
import numpy as np
import pandas as pd

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vega, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# --------------------------------------------------------------------------------
# Making dataframe
# --------------------------------------------------------------------------------
np.random.seed(4)
DATA_FRAME = None


def fetch_data(samples=15):
    global DATA_FRAME
    deltas = cycle(
        [
            pd.Timedelta(weeks=-2),
            pd.Timedelta(days=-1),
            pd.Timedelta(hours=-1),
            pd.Timedelta(0),
            pd.Timedelta(minutes=5),
            pd.Timedelta(seconds=10),
            pd.Timedelta(microseconds=50),
            pd.Timedelta(microseconds=10),
        ]
    )
    dummy_data = {
        "id": range(samples),
        "date_time_naive": pd.date_range("2021-01-01", periods=samples),
        "apple": np.random.randint(0, 100, samples) / 3.0,
        "banana": np.random.randint(0, 100, samples) / 5.0,
        "chocolate": np.random.randint(0, 100, samples),
        "group": np.random.choice(["A", "B"], size=samples),
        "season": np.random.choice(
            ["Spring", "Summer", "Fall", "Winter"], size=samples
        ),
        "date_only": pd.date_range("2020-01-01", periods=samples).date,
        "timedelta": [next(deltas) for i in range(samples)],
        "date_tz_aware": pd.date_range(
            "2022-01-01", periods=samples, tz="Asia/Katmandu"
        ),
    }
    DATA_FRAME = pd.DataFrame(dummy_data)
    return DATA_FRAME


fetch_data()

# --------------------------------------------------------------------------------
# Preparing table
# --------------------------------------------------------------------------------
header_options = {"apple": {"sortable": False}}
headers, rows = vuetify.dataframe_to_grid(DATA_FRAME, header_options)

table = {
    "headers": ("headers", headers),
    "items": ("rows", rows),
    "v_model": ("selection", []),
    "search": ("query", ""),
    "classes": "elevation-1 ma-4",
    "multi_sort": True,
    "dense": True,
    "show_select": True,
    "single_select": False,
    "item_key": "id",
}

# --------------------------------------------------------------------------------
# Describing chart
# --------------------------------------------------------------------------------


@state.change("selection")
def selection_change(selection=[], **kwargs):
    global DATA_FRAME
    selected_df = pd.DataFrame(selection)

    # Chart
    chart_data = DATA_FRAME.loc[
        :, ["date_time_naive", "apple", "banana", "chocolate"]
    ].assign(source="total")

    if not selected_df.empty:
        selected_data = selected_df.loc[
            :, ["date_time_naive", "apple", "banana", "chocolate"]
        ].assign(source="selection")
        chart_data = pd.concat([chart_data, selected_data])

    chart_data = pd.melt(
        chart_data,
        id_vars=["date_time_naive", "source"],
        var_name="item",
        value_name="quantity",
    )
    chart = (
        alt.Chart(chart_data)
        .mark_bar()
        .encode(
            x=alt.X("item:O"),
            y=alt.Y("sum(quantity):Q", stack=False),
            color=alt.Color("source:N", scale=alt.Scale(domain=["total", "selection"])),
        )
    ).properties(width="container", height=100)

    ctrl.fig_update(chart)


# --------------------------------------------------------------------------------
# GUI
# --------------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.title.set_text("Vuetify table example")
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VTextField(
            v_model=("query",),
            placeholder="Search",
            dense=True,
            hide_details=True,
            prepend_icon="mdi-magnify",
        )

    with layout.content:
        with vuetify.VRow(classes="justify-center ma-6"):
            fig = vega.Figure(classes="ma-2", style="width: 100%;")
            ctrl.fig_update = fig.update
        vuetify.VDataTable(**table)

# -----------------------------------------------------------------------------
# Start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from paraview import servermanager, simple

from trame.app import get_server
from trame.decorators import TrameApp, change
from trame.ui.vuetify3 import SinglePageWithDrawerLayout
from trame.widgets import paraview as pv_widgets
from trame.widgets import vuetify3 as v3


@TrameApp()
class App:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue3")
        self.state.volume = 0

        # Data source
        self.source = simple.Wavelet()
        self.source.UpdatePipeline()

        # Extract info
        ds = servermanager.Fetch(self.source)
        self.origin = ds.GetOrigin()
        self.spacing = ds.GetSpacing()
        self.extent = ds.GetExtent()

        # Pipeline
        self.extract = simple.ExtractSubset(
            Input=self.source,
            VOI=list(self.extent),
        )
        self.representation = simple.Show(self.extract)
        self.representation.SetRepresentationType("Volume")
        self.view = simple.Render()

        # bbox
        self.box = simple.Box()
        self.box_rep = simple.Show(self.box)
        self.box_rep.SetRepresentationType("Wireframe")

        self._build_ui()

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    @change("i_range", "j_range", "k_range")
    def update_roi(self, i_range, j_range, k_range, **_):
        # Update Volume
        self.extract.VOI = [*i_range, *j_range, *k_range]

        # Update box
        self.box.XLength = self.spacing[0] * (i_range[1] - i_range[0])
        self.box.YLength = self.spacing[1] * (j_range[1] - j_range[0])
        self.box.ZLength = self.spacing[2] * (k_range[1] - k_range[0])
        self.box.Center = [
            self.origin[0] + 0.5 * self.spacing[0] * (i_range[1] + i_range[0]),
            self.origin[1] + 0.5 * self.spacing[0] * (j_range[1] + j_range[0]),
            self.origin[2] + 0.5 * self.spacing[0] * (k_range[1] + k_range[0]),
        ]

        # Compute volume
        self.extract.UpdatePipeline()
        bounds = self.extract.GetDataInformation().GetBounds()
        self.state.volume = (
            (bounds[1] - bounds[0]) * (bounds[3] - bounds[2]) * (bounds[5] - bounds[4])
        )

        self.ctrl.view_update()

    def _build_ui(self):
        with SinglePageWithDrawerLayout(self.server, full_height=True) as layout:
            with layout.toolbar:
                v3.VLabel(
                    "Extent [{{i_range.join(', ')}}, {{j_range.join(', ')}}, {{k_range.join(', ')}}]"
                    " - Volume: {{ volume }}",
                    classes="mx-2",
                )
                v3.VBtn(icon="mdi-crop-free", click=self.ctrl.view_reset_camera)
            with layout.drawer as drawer:
                drawer.width = 300
                with v3.VCol():
                    v3.VRangeSlider(
                        v_model=("i_range", self.extent[0:2]),
                        min=-10,
                        max=10,
                        step=1,
                        density="compact",
                        hide_details=True,
                    )
                    v3.VRangeSlider(
                        v_model=("j_range", self.extent[2:4]),
                        min=-10,
                        max=10,
                        step=1,
                        density="compact",
                        hide_details=True,
                    )
                    v3.VRangeSlider(
                        v_model=("k_range", self.extent[4:]),
                        min=-10,
                        max=10,
                        step=1,
                        density="compact",
                        hide_details=True,
                    )

            with layout.content:
                with v3.VContainer(fluid=True, classes="fill-height pa-0"):
                    with pv_widgets.VtkRemoteView(
                        self.view,
                        interactive_ratio=1,
                    ) as view:
                        self.ctrl.view_update = view.update
                        self.ctrl.view_reset_camera = view.reset_camera


def main():
    app = App()
    app.server.start()


if __name__ == "__main__":
    main()



from trame.app import get_server
from trame.app.file_upload import ClientFile
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import vuetify

server = get_server(client_type="vue2")

with SinglePageWithDrawerLayout(server) as layout:
    with layout.toolbar:
        vuetify.VFileInput(
            v_model=("file_exchange", None),
        )


@server.state.change("file_exchange")
def handle(file_exchange, **kwargs):
    file = ClientFile(file_exchange)
    print(file.info)


if __name__ == "__main__":
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/PlainPython/ClientOnlyVTK/app.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/33f52b6bb9eb73129b181699a94be9ad86187d49

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

state.trame__title = "VTK Rendering"

with SinglePageLayout(server) as layout:
    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            with vtk_widgets.VtkView(ref="view"):
                with vtk_widgets.VtkGeometryRepresentation():
                    vtk_widgets.VtkAlgorithm(
                        vtk_class="vtkConeSource", state=("{ resolution }",)
                    )

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            hide_details=True,
            v_model=("resolution", 6),
            max=60,
            min=3,
            step=1,
            style="max-width: 300px;",
        )
        vuetify.VSwitch(
            hide_details=True,
            v_model=("$vuetify.theme.dark",),
        )
        with vuetify.VBtn(icon=True, click="$refs.view.resetCamera()"):
            vuetify.VIcon("mdi-crop-free")

if __name__ == "__main__":
    server.start()



import os

from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import trame, vuetify3

server = get_server()
server.client_type = os.environ.get("VUE_VERSION", "vue3")

FILE_LISTING = [
    {
        "text": "Hello.txt",
        "value": "hello.txt",
        "type": "File",
        "prependIcon": "mdi-file-document-outline",
    }
]

PATH_HIERARCHY = []

server.state["menu_items"] = ["one", "two", "three"]


def on_click(e):
    print(e)


def load_data(entry, item):
    print(entry, item)


with SinglePageLayout(server) as layout:
    layout.title.set_text("List Browser")
    with layout.content:
        trame.ListBrowser(
            style="position: relative",
            classes="pa-8",
            location=("[100, 100]",),
            handle_position="bottom",
            filter=True,
            list=("listing", FILE_LISTING),
            path=("path", PATH_HIERARCHY),
            click=(on_click, "[$event]"),
            contextmenu="""
                    $event.preventDefault();
                    menuX = $event.clientX;
                    menuY = $event.clientY;
                    menuShow = $event.srcElement.textContent;
                 """,
        )
        with vuetify3.VMenu(
            v_model=("menuShow", False),
            style=("{ position: 'absolute', left: `${menuX}px`, top: `${menuY}px`}",),
        ) as menu:
            with vuetify3.VList():
                with vuetify3.VListItem(
                    v_for="(item, i) in menu_items",
                    key="i",
                    value=["item"],
                ):
                    vuetify3.VBtn(
                        "{{ item }}",
                        click=(load_data, "[menuShow, item]"),
                    )


if __name__ == "__main__":
    server.start()



from trame.app import TrameApp
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify3 as v3


class Cone(TrameApp):
    def __init__(self, server_or_name=None):
        super().__init__(server_or_name)
        self._build_ui()

    @property
    def resolution(self):
        return self.state.resolution

    @resolution.setter
    def resolution(self, v):
        with self.state:
            self.state.resolution = v

    def reset_resolution(self):
        self.resolution = 6

    def _build_ui(self):
        with SinglePageLayout(self.server) as self.ui:
            self.ui.title.set_text("Trame demo")
            with self.ui.toolbar as toolbar:
                toolbar.density = "compact"
                v3.VSpacer()
                v3.VSlider(
                    v_model=("resolution", 6),
                    min=3,
                    max=60,
                    step=1,
                    hide_details=True,
                    style="max-width: 300px;",
                )
                v3.VBtn(icon="mdi-lock-reset", click=self.reset_resolution)
                v3.VBtn(icon="mdi-crop-free", click=self.ctrl.view_reset_camera)

            with self.ui.content:
                with v3.VContainer(fluid=True, classes="pa-0 fill-height"):
                    with vtk_widgets.VtkView() as view:
                        self.ctrl.view_reset_camera = view.reset_camera
                        with vtk_widgets.VtkGeometryRepresentation():
                            vtk_widgets.VtkAlgorithm(
                                vtk_class="vtkConeSource",
                                state=("{ resolution }",),
                            )


def main(**kwargs):
    cone = Cone()
    cone.server.start(**kwargs)


if __name__ == "__main__":
    main()



import paraview.web.venv  # Available in PV 5.10
from paraview import simple

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import paraview, vuetify

# -----------------------------------------------------------------------------
# trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# ParaView code
# -----------------------------------------------------------------------------

DEFAULT_RESOLUTION = 6

cone = simple.Cone()
representation = simple.Show(cone)
view = simple.Render()


@state.change("resolution")
def update_cone(resolution, **kwargs):
    cone.Resolution = resolution
    ctrl.view_update()


def update_reset_resolution():
    state.resolution = DEFAULT_RESOLUTION


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

state.trame__title = "ParaView cone"

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("Cone Application")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        vuetify.VDivider(vertical=True, classes="mx-2")
        with vuetify.VBtn(icon=True, click=update_reset_resolution):
            vuetify.VIcon("mdi-undo-variant")

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            html_view = paraview.VtkRemoteView(view)
            # html_view = paraview.VtkLocalView(view)
            ctrl.view_update = html_view.update
            ctrl.view_reset_camera = html_view.reset_camera

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



"""
This module is deprecated and is currently replaced by existing API living on the trame layout.

In Jupyter you just need to do the following to display it to the output cell:

>>> await layout.ready
>>> layout
"""



from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout
from trame.widgets import vuetify3 as vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server()
state, ctrl = server.state, server.controller

state.menu_items = ["one", "two", "three"]


def print_item(item):
    print("Clicked on", item)


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

state.trame__title = "Menu example"

with SinglePageLayout(server) as layout:
    with layout.toolbar:
        vuetify.VSpacer()
        with vuetify.VMenu():
            with vuetify.Template(v_slot_activator="{ on, attrs }"):
                with vuetify.VBtn(icon=True, v_bind="attrs", v_on="on"):
                    vuetify.VIcon("mdi-dots-vertical")
            with vuetify.VList():
                with vuetify.VListItem(
                    v_for="(item, i) in menu_items",
                    key="i",
                    value=["item"],
                ):
                    vuetify.VBtn(
                        "{{ item }}",
                        click=(print_item, "[item]"),
                    )

if __name__ == "__main__":
    server.start()



import sys
from trame.app import TrameApp
from trame.ui.html import DivLayout
from trame.widgets import html


def get_rendering_information():
    try:
        import vtk

        renderer = vtk.vtkRenderer()
        render_window = vtk.vtkRenderWindow()
        render_window.AddRenderer(renderer)
        render_window.Render()

        return render_window.ReportCapabilities()

    except ImportError as err:
        return err.msg


class VtkRenderingInfo(TrameApp):
    def __init__(self, server=None):
        super().__init__(server)

        with DivLayout(self.server) as self.ui:
            html.Pre("{{ report }}")

        self.state.report = get_rendering_information()


def main():
    app = VtkRenderingInfo()
    app.server.start()


if __name__ == "__main__":
    if "--stdout" in sys.argv:
        print(get_rendering_information())
    else:
        main()



import plotly.express as px
import plotly.graph_objects as go

from trame.app import get_server
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import plotly, vuetify

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


def contour_plot():
    """https://plotly.com/python/contour-plots/"""
    return go.Figure(
        data=go.Contour(
            z=[
                [10, 10.625, 12.5, 15.625, 20],
                [5.625, 6.25, 8.125, 11.25, 15.625],
                [2.5, 3.125, 5.0, 8.125, 12.5],
                [0.625, 1.25, 3.125, 6.25, 10.625],
                [0, 0.625, 2.5, 5.625, 10],
            ]
        )
    )


def bar_plot(color="Gold"):
    return go.Figure(data=go.Bar(y=[2, 3, 1], marker_color=color))


def scatter():
    df = px.data.iris()

    fig = px.scatter(
        df,
        x="sepal_width",
        y="sepal_length",
        color="species",
        title="Using The add_trace() method With A Plotly Express Figure",
    )

    fig.add_trace(
        go.Scatter(
            x=[2, 4],
            y=[4, 8],
            mode="lines",
            line=go.scatter.Line(color="gray"),
            showlegend=False,
        )
    )

    return fig


card_config = {}

card_config["plots"] = {
    "type": "figure",
    "plot_list": [
        {"text": "Contour", "value": contour_plot},
        {"text": "Bar", "value": bar_plot},
        {"text": "Scatter", "value": scatter},
    ],
    "title": "Plots",
}

card_config["else"] = {
    "type": "figure",
    "plot_list": [
        {"text": "Bar", "value": bar_plot},
        {"text": "Contour", "value": contour_plot},
        {"text": "Scatter", "value": scatter},
    ],
    "title": "Else",
}


def on_event(type, e):
    print(type, e)


def _ui_card(title, ui_name):
    with vuetify.VCard(v_show=f"active_ui == '{ui_name}'"):
        vuetify.VCardTitle(
            title,
            classes="grey lighten-1 py-1 grey--text text--darken-3",
            style="user-select: none; cursor: pointer",
            hide_details=True,
            dense=True,
        )
        content = vuetify.VCardText(classes="py-2")
    return content


def add_card(key, value):
    with _ui_card(value["title"], key):
        with vuetify.VList(rounded=True):
            with vuetify.VListItemGroup(
                v_model=("active_plot", "Contour"), color="primary", mandatory=True
            ):
                for key in value["plot_list"]:
                    with vuetify.VListItem(value=key["text"]):
                        with vuetify.VListItemContent():
                            vuetify.VListItemTitle(key["text"])


@state.change("active_plot")
def update_plot(active_plot, **kwargs):
    if state.active_ui:
        for item in card_config[state.active_ui]["plot_list"]:
            if item["text"] == active_plot:
                ctrl.figure_update(item["value"]())


@state.change("visualization_mode")
def __update_mode(visualization_mode, **kwargs):
    for key in card_config.keys():
        if visualization_mode == key:
            state.active_ui = key
    state.active_plot = card_config[state.active_ui]["plot_list"][0]["text"]
    update_plot(state.active_plot)


state.trame__title = "Plotly"

with SinglePageWithDrawerLayout(server) as layout:
    layout.title.set_text("trame ❤️ plotly")

    with layout.drawer:
        with vuetify.VRadioGroup(
            v_model=("visualization_mode", next(iter(card_config))),
            classes="ml-4",
            height=100,
        ):
            for key, value in card_config.items():
                vuetify.VRadio(label=value["title"], value=key)
        for key, value in card_config.items():
            add_card(key, value)
    with layout.content:
        with vuetify.VContainer(fluid=True):
            with vuetify.VRow(dense=True):
                vuetify.VSpacer()
                figure = plotly.Figure(
                    display_logo=False,
                    display_mode_bar="true",
                    # click=(on_event, "['selected', utils.safe($event)]"),
                    # hover=(on_event, "['hover', utils.safe($event)]"),
                    # selecting=(on_event, "['selecting', $event]"),
                    # unhover=(on_event, "['unhover', $event]"),
                )
                ctrl.figure_update = figure.update
                vuetify.VSpacer()

if __name__ == "__main__":
    server.start()



import vtkmodules.vtkRenderingOpenGL2  # noqa

# -----------------------------------------------------------------------------
# VTK
# -----------------------------------------------------------------------------
from vtkmodules.vtkFiltersSources import vtkConeSource
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import html, trame, vtk, vuetify

DEFAULT_RESOLUTION = 6

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)
renderWindow.OffScreenRenderingOn()

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

cone_source = vtkConeSource()
mapper = vtkPolyDataMapper()
actor = vtkActor()
mapper.SetInputConnection(cone_source.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


def reset_resolution():
    state.resolution = 6


@state.change("resolution")
def update_resolution(resolution, **kwargs):
    cone_source.SetResolution(int(resolution))
    ctrl.view_update()


# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

layout = SinglePageLayout(server)

with layout:
    # Validate client life cycle
    trame.LifeCycleMonitor(events=("['created']",))

    layout.icon.click = ctrl.reset_camera
    layout.title.set_text("Cone")
    layout.toolbar.dense = True

    # Toolbar
    with layout.toolbar as toolbar:
        vuetify.VSpacer()
        html.Div("Test size", classes="text-h6")
        vuetify.VSlider(
            hide_details=True,
            v_model=("resolution", 6),
            max=60,
            min=3,
            step=1,
            style="max-width: 300px;",
        )
        vuetify.VSwitch(
            hide_details=True,
            v_model=("$vuetify.theme.dark",),
        )
        with vuetify.VBtn(icon=True, click=reset_resolution):
            vuetify.VIcon("mdi-undo")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            view = vtk.VtkRemoteView(renderWindow, interactive_ratio=1)
            ctrl.view_update = view.update
            ctrl.reset_camera = view.reset_camera


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start(exec_mode="desktop")



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/SimpleCone/RemoteRendering.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/674f72774228bbcab5689417c1c5642230b1eab8

Installation requirements:
    pip install trame trame-vuetify trame-vtk vtk
"""

import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersSources import vtkConeSource

# VTK factory initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "VTK Remote rendering"

# -----------------------------------------------------------------------------
# Custom / Advanced event handling
# -----------------------------------------------------------------------------

VTK_VIEW_EVENTS = [
    "StartAnimation",
    "Animation",
    "EndAnimation",
    "MouseEnter",
    "MouseLeave",
    "StartMouseMove",
    "MouseMove",
    "EndMouseMove",
    "LeftButtonPress",
    "LeftButtonRelease",
    "MiddleButtonPress",
    "MiddleButtonRelease",
    "RightButtonPress",
    "RightButtonRelease",
    "KeyPress",
    "KeyDown",
    "KeyUp",
    "StartMouseWheel",
    "MouseWheel",
    "EndMouseWheel",
    "StartPinch",
    "Pinch",
    "EndPinch",
    "StartPan",
    "Pan",
    "EndPan",
    "StartRotate",
    "Rotate",
    "EndRotate",
    "Button3D",
    "Move3D",
    "StartPointerLock",
    "EndPointerLock",
    "StartInteraction",
    "Interaction",
    "EndInteraction",
]
DEFAULT_RESOLUTION = 6


def on_event(*args, **kwargs):
    print("event", args, kwargs)


def event_listeners(events):
    result = {}
    for event in events:
        result[event] = (on_event, "[utils.vtk.event($event)]")
    return result


# -----------------------------------------------------------------------------
# VTK code
# -----------------------------------------------------------------------------

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)
renderWindow.OffScreenRenderingOn()

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

cone_source = vtkConeSource()
mapper = vtkPolyDataMapper()
actor = vtkActor()
mapper.SetInputConnection(cone_source.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()


@state.change("resolution")
def update_cone(resolution=DEFAULT_RESOLUTION, **kwargs):
    cone_source.SetResolution(resolution)
    ctrl.view_update()


def update_reset_resolution():
    state.resolution = DEFAULT_RESOLUTION


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("Cone Application")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        vuetify.VDivider(vertical=True, classes="mx-2")
        with vuetify.VBtn(icon=True, click=update_reset_resolution):
            vuetify.VIcon("mdi-undo-variant")

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            view = vtk_widgets.VtkRemoteView(
                renderWindow,
                ref="view",
                # For Custom / Advanced event handling
                # interactor_events=("event_types", VTK_VIEW_EVENTS),
                # **event_listeners(VTK_VIEW_EVENTS),
            )
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera

    # Uncomment following line to hide footer
    # layout.footer.hide()


# -----------------------------------------------------------------------------
# Jupyter
# -----------------------------------------------------------------------------


def show(**kwargs):
    from trame.app import jupyter

    jupyter.show(server, **kwargs)


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import helper, html

# From: https://quasar.dev/start/umd

module = dict(
    scripts=[
        "https://cdn.jsdelivr.net/npm/quasar@2.11.5/dist/quasar.umd.prod.js",
    ],
    styles=[
        "https://fonts.googleapis.com/css?family=Roboto:100,300,400,500,700,900|Material+Icons",
        "https://cdn.jsdelivr.net/npm/quasar@2.11.5/dist/quasar.prod.css",
    ],
    vue_use=[
        "Quasar",
    ],
)

QSlider = helper.create_class(
    "QSlider",
    "q-slider",
    module=module,
    properties=[
        "min",
        "max",
    ],
)

QBtn = helper.create_class(
    "QBtn",
    "q-btn",
    module=module,
    properties=[
        "label",
    ],
    events=[
        "click",
    ],
)

QCircularProgress = helper.create_class(
    "QCircularProgress",
    "q-circular-progress",
    module=module,
    properties=[
        "value",
        "indeterminate",
        "size",
        "thickness",
        "color",
        "center_color",
    ],
)

# -----------------------------------------------------------------------------
# Trame usage
# -----------------------------------------------------------------------------

server = get_server(client_type="vue3")


def reset():
    server.state.value = 5


with DivLayout(server) as layout:
    with html.Div(classes="q-pa-md"):
        with html.Div(classes="row items-center"):
            html.Div("{{ value }}", classes="col-2")
            QBtn(label="Hello", classes="col", click=reset)
            QCircularProgress(
                indeterminate=True,
                size="75px",
                thickness=0.6,
                color="lime",
                center_color="grey-8",
                classes="q-ma-md col",
            )
            QSlider(
                v_model_number=("value", 0),
                min=("1",),
                max=("100",),
                step=("1",),
                classes="col",
            )
            QCircularProgress(
                size="75px",
                thickness=0.6,
                color="lime",
                center_color="grey-8",
                classes="q-ma-md col",
                value=("value",),
            )

server.start()



from pathlib import Path

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html

BINARY_FILE = Path(__file__).parent.parent / "data/can.ex2"

server = get_server()
state, ctrl = server.state, server.controller


@ctrl.trigger("download_binary")
def download():
    return server.protocol.addAttachment(BINARY_FILE.read_bytes())


with DivLayout(server):
    html.Button(
        "Download ",
        click="utils.download('can.ex2', trigger('download_binary'), 'application/octet-stream')",
    )

if __name__ == "__main__":
    server.start()



import time

import trame.widgets.vuetify3 as v3
from trame.app import asynchronous, get_server
from trame.ui.vuetify3 import VAppLayout

# -------------------------------------------------------------------
# Helper
# -------------------------------------------------------------------


class AsyncTracker:
    def __init__(self, server, name):
        self.name = name
        self.server = server
        self.state = server.state

    async def __aenter__(self):
        with self.state:
            self.state[self.name] = True
        await self.server.network_completion

    async def __aexit__(self, exc_t, exc_v, exc_tb):
        with self.state:
            self.state[self.name] = False
        await self.server.network_completion


# -------------------------------------------------------------------


def long_task():
    time.sleep(2)
    print("Finished")


async def run_long_task():
    async with AsyncTracker(server, "loading"):
        long_task()


def run_task():
    asynchronous.create_task(run_long_task())


# -------------------------------------------------------------------

server = get_server()
state = server.state

with VAppLayout(server) as layout:
    with layout:
        with v3.Template():
            with v3.VOverlay(
                v_model=("loading", False),
                classes="align-center justify-center",
                close_on_content_click=False,
                persistent=True,
                style="z-index: 10000;",
            ):
                v3.VProgressCircular(color="primary", size=64, indeterminate=True)
        v3.VBtn("Long task (no tracking)", click=long_task)
        v3.VBtn("Long task (tracking)", click=run_task)


if __name__ == "__main__":
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/Applications/FiniteElementAnalysis/app_remote_view.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/03f28bb0084490acabf218264b96a1dbb3a17f19

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

import io
import os

import numpy as np
import pandas as pd
import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.numpy_interface.dataset_adapter import numpyTovtkDataArray as np2da
from vtkmodules.util import vtkConstants
from vtkmodules.vtkCommonCore import vtkIdList, vtkLookupTable, vtkPoints
from vtkmodules.vtkCommonDataModel import vtkCellArray, vtkUnstructuredGrid
from vtkmodules.vtkFiltersCore import vtkThreshold

# Add import for the rendering
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkDataSetMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

vtk_idlist = vtkIdList()
vtk_grid = vtkUnstructuredGrid()
vtk_filter = vtkThreshold()
vtk_filter.SetInputData(vtk_grid)
field_to_keep = "my_array"

renderer = vtkRenderer()
renderer.SetBackground(0.8, 0.8, 0.8)
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

filter_mapper = vtkDataSetMapper()
filter_mapper.SetInputConnection(vtk_filter.GetOutputPort())
filter_actor = vtkActor()
filter_actor.SetMapper(filter_mapper)
renderer.AddActor(filter_actor)

lut = vtkLookupTable()
lut.SetHueRange(0.667, 0)
lut.Build()
filter_mapper.SetLookupTable(lut)

mesh_mapper = vtkDataSetMapper()
mesh_mapper.SetInputData(vtk_grid)
mesh_mapper.SetScalarVisibility(0)
mesh_actor = vtkActor()
mesh_actor.SetMapper(mesh_mapper)
renderer.AddActor(mesh_actor)


@state.change("nodes_file", "elems_file", "field_file")
def update_grid(nodes_file, elems_file, field_file, **kwargs):
    if not nodes_file:
        return

    if not elems_file:
        return

    nodes_bytes = nodes_file.get("content")
    elems_bytes = elems_file.get("content")

    if isinstance(nodes_bytes, list):
        nodes_bytes = b"".join(nodes_bytes)

    if isinstance(elems_bytes, list):
        elems_bytes = b"".join(elems_bytes)

    df_nodes = pd.read_csv(
        io.StringIO(nodes_bytes.decode("utf-8")),
        delim_whitespace=True,
        header=None,
        skiprows=1,
        names=["id", "x", "y", "z"],
    )

    df_nodes["id"] = df_nodes["id"].astype(int)
    df_nodes = df_nodes.set_index("id", drop=True)
    # fill missing ids in range as VTK uses position (index) to map cells to points
    df_nodes = df_nodes.reindex(
        np.arange(df_nodes.index.min(), df_nodes.index.max() + 1), fill_value=0
    )

    df_elems = pd.read_csv(
        io.StringIO(elems_bytes.decode("utf-8")),
        skiprows=1,
        header=None,
        delim_whitespace=True,
        engine="python",
        index_col=None,
    ).sort_values(0)
    # order: 0: eid, 1: eshape, 2+: nodes, iloc[:,0] is index
    df_elems.iloc[:, 0] = df_elems.iloc[:, 0].astype(int)

    n_nodes = df_elems.iloc[:, 1].map(
        lambda x: int("".join(i for i in x if i.isdigit()))
    )
    df_elems.insert(2, "n_nodes", n_nodes)
    # fill missing ids in range as VTK uses position (index) to map data to cells
    new_range = np.arange(df_elems.iloc[:, 0].min(), df_elems.iloc[:, 0].max() + 1)
    df_elems = df_elems.set_index(0, drop=False).reindex(new_range, fill_value=0)

    # mapping specific to Ansys Mechanical data
    vtk_shape_id_map = {
        "Tet4": vtkConstants.VTK_TETRA,
        "Tet10": vtkConstants.VTK_QUADRATIC_TETRA,
        "Hex8": vtkConstants.VTK_HEXAHEDRON,
        "Hex20": vtkConstants.VTK_QUADRATIC_HEXAHEDRON,
        "Tri6": vtkConstants.VTK_QUADRATIC_TRIANGLE,
        "Quad8": vtkConstants.VTK_QUADRATIC_QUAD,
        "Tri3": vtkConstants.VTK_TRIANGLE,
        "Quad4": vtkConstants.VTK_QUAD,
        "Wed15": vtkConstants.VTK_QUADRATIC_WEDGE,
    }
    df_elems["cell_types"] = np.nan
    df_elems.loc[df_elems.loc[:, 0] > 0, "cell_types"] = df_elems.loc[
        df_elems.loc[:, 0] > 0, 1
    ].map(
        lambda x: (
            vtk_shape_id_map[x.strip()]
            if x.strip() in vtk_shape_id_map.keys()
            else np.nan
        )
    )
    df_elems = df_elems.dropna(subset=["cell_types"], axis=0)

    # convert dataframes to vtk-desired format
    points = df_nodes[["x", "y", "z"]].to_numpy()
    cell_types = df_elems["cell_types"].to_numpy()
    n_nodes = df_elems.loc[:, "n_nodes"].to_numpy()
    # subtract starting node id from all grid references in cells to avoid
    # filling from 0 to first used node (in case mesh doesn't start at 1)
    p = df_elems.iloc[:, 3:-1].to_numpy() - df_nodes.index.min()
    # if you need to, re-order nodes here-ish
    a = np.hstack((n_nodes.reshape((len(n_nodes), 1)), p))
    # convert to flat numpy array
    cells = a.ravel()
    # remove nans (due to elements with different no. of nodes)
    cells = cells[np.logical_not(np.isnan(cells))]
    cells = cells.astype(int)

    # update grid
    vtk_pts = vtkPoints()
    vtk_pts.SetData(np2da(points))
    vtk_grid.SetPoints(vtk_pts)

    vtk_cells = vtkCellArray()
    vtk_cells.SetCells(
        cell_types.shape[0], np2da(cells, array_type=vtkConstants.VTK_ID_TYPE)
    )
    vtk_grid.SetCells(
        np2da(cell_types, array_type=vtkConstants.VTK_UNSIGNED_CHAR), vtk_cells
    )
    state.mesh_status = 1

    # Add field if any
    if field_file:
        field_bytes = field_file.get("content")
        if isinstance(field_bytes, list):
            field_bytes = b"".join(field_bytes)
        df_elem_data = pd.read_csv(
            io.StringIO(field_bytes.decode("utf-8")),
            delim_whitespace=True,
            header=None,
            skiprows=1,
            names=["id", "val"],
        )
        df_elem_data = df_elem_data.sort_values("id").set_index("id", drop=True)
        # fill missing ids in range as VTK uses position (index) to map data to cells
        df_elem_data = df_elem_data.reindex(
            np.arange(df_elems.index.min(), df_elems.index.max() + 1), fill_value=0.0
        )
        np_val = df_elem_data["val"].to_numpy()
        # assign data to grid with the name 'my_array'
        vtk_array = np2da(np_val, name=field_to_keep)
        vtk_grid.GetCellData().SetScalars(vtk_array)
        full_min, full_max = vtk_array.GetRange()
        state.full_min = full_min
        state.full_max = full_max
        state.threshold_range = [full_min, full_max]
        state.mesh_status = 2

        # Color handling in plain VTK
        filter_mapper.SetScalarRange(full_min, full_max)

        # Write dataset as VTU
        # from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridWriter
        # writer = vtkXMLUnstructuredGridWriter()
        # writer.SetFileName("fea.vtu")
        # writer.SetInputData(vtk_grid)
        # writer.SetCompressorTypeToZLib()
        # writer.SetCompressionLevel(6)
        # writer.SetDataModeToAppended()
        # writer.Write()

    renderer.ResetCamera()
    ctrl.view_update()


@state.change("threshold_range")
def update_filter(threshold_range, **kwargs):
    # Comment if you want to have a fix color range
    filter_mapper.SetScalarRange(threshold_range)
    vtk_filter.SetLowerThreshold(threshold_range[0])
    vtk_filter.SetUpperThreshold(threshold_range[1])
    ctrl.view_update()


@state.change("mesh_status")
def update_mesh_representations(**kwargs):
    # defaults
    color = [1, 1, 1]
    representation = 2
    opacity = 1

    if state.mesh_status == 2:
        color = [0.3, 0.3, 0.3]
        representation = 1
        opacity = 0.2

    property = mesh_actor.GetProperty()
    property.SetRepresentation(representation)
    property.SetColor(color)
    property.SetOpacity(opacity)
    property.SetLineWidth(2)
    ctrl.view_update()


def reset():
    state.update(
        {
            "nodes_file": None,
            "elems_file": None,
            "field_file": None,
            "mesh_status": 0,
        }
    )


# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------
file_style = {
    "dense": True,
    "hide_details": True,
    "style": "max-width: 200px",
    "class": "mx-2",
    "small_chips": True,
    "clearable": ("false",),
    "accept": ".txt",
}

state.trame__title = "FEA - Mesh viewer"


with SinglePageLayout(server) as layout:
    layout.icon.click = reset
    layout.title.set_text("Mesh Viewer")

    # Toolbar ----------------------------------------
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VRangeSlider(
            thumb_size=16,
            thumb_label=True,
            label="Threshold",
            v_if=("mesh_status > 1",),
            v_model=("threshold_range", [0, 1]),
            min=("full_min", 0),
            max=("full_max", 1),
            dense=True,
            hide_details=True,
            style="max-width: 400px",
        )
        vuetify.VFileInput(
            v_show=("mesh_status < 1",),
            prepend_icon="mdi-vector-triangle",
            v_model=("nodes_file", None),
            placeholder="Nodes",
            **file_style,
        )
        vuetify.VFileInput(
            v_show=("mesh_status < 1",),
            prepend_icon="mdi-dots-triangle",
            v_model=("elems_file", None),
            placeholder="Elements",
            **file_style,
        )
        vuetify.VFileInput(
            v_show=("mesh_status < 2",),
            prepend_icon="mdi-gradient",
            v_model=("field_file", None),
            placeholder="Field",
            **file_style,
        )
        with vuetify.VBtn(
            v_if=("mesh_status",), icon=True, click=ctrl.view_reset_camera
        ):
            vuetify.VIcon("mdi-crop-free")

        vuetify.VProgressLinear(
            indeterminate=True, absolute=True, bottom=True, active=("trame__busy",)
        )

    # Content ----------------------------------------
    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
            style="position: relative",
        ):
            html_view = vtk_widgets.VtkRemoteView(
                renderWindow, interactive_ratio=("1",), interactive_quality=(80,)
            )
            ctrl.view_update = html_view.update
            ctrl.view_reset_camera = html_view.reset_camera


# Variables not defined within HTML but used
state.mesh_status = 0  # 0: empty / 1: mesh / 2: mesh+filter

# -----------------------------------------------------------------------------
# Use --data to skip file upload
# -----------------------------------------------------------------------------

parser = server.cli
parser.add_argument("--data", help="Unstructured file path", dest="data")
args = parser.parse_args()
if args.data:
    from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridReader

    reader = vtkXMLUnstructuredGridReader()
    reader.SetFileName(os.path.abspath(args.data))
    reader.Update()
    vtu = reader.GetOutput()
    vtk_grid.ShallowCopy(vtu)

    vtk_array = vtu.GetCellData().GetScalars()
    full_min, full_max = vtk_array.GetRange()
    state.full_min = full_min
    state.full_max = full_max
    state.threshold_range = [full_min, full_max]
    state.mesh_status = 2
    update_mesh_representations()

# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



# -----------------------------------------------------------------------------
# trame
# -----------------------------------------------------------------------------
from trame.app import get_server
from trame.widgets import html

CLIENT_TYPE = "vue3"

if CLIENT_TYPE == "vue2":
    from trame.ui.vuetify2 import SinglePageLayout
    from trame.widgets import vuetify2 as vuetify
else:
    from trame.ui.vuetify3 import SinglePageLayout
    from trame.widgets import vuetify3 as vuetify

server = get_server()
server.client_type = CLIENT_TYPE

state, ui = server.state, server.ui  # ui is an instance of VirtualNodeManager


def my_component(name):
    """Emulate a component that is discovered at runtime"""
    vname = f"my_comp_val_{name}"
    with state:  # Force to update state before template
        state.setdefault(vname, 0)
    html.Div(name)
    vuetify.VBtn("{{" + vname + "}}", click=f"{vname}++")


def udpate_content():
    with ui.customizable_slot.clear():
        my_component(state.demanded_name)


# -----------------------------------------------------------------------------
# GUI (Layout)
# -----------------------------------------------------------------------------

# virtualnodes added to layout
with SinglePageLayout(server) as layout:
    with layout.content as content:
        content.clear()
        vuetify.VTextField(v_model=("demanded_name", "init"))
        vuetify.VBtn("update content", click=udpate_content)
        with html.Div():
            ui.customizable_slot(layout)


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start(open_browser=False)



r"""
Installation requirements:
    pip install trame trame-vuetify trame-components
"""

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import trame

server = get_server(client_type="vue2")

with SinglePageLayout(server) as layout:
    layout.title.set_text("FloatCard Demo")

    with layout.content:
        trame.FloatCard(
            "Drag the handle to move me anywhere",
            classes="pa-8",
        )

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify pandas
"""

import json

import pandas as pd

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vuetify

server = get_server(client_type="vue2")

# --------------------------------------------------------------------------------
# Loading dataframe
# --------------------------------------------------------------------------------
data = json.loads(
    """
[
    {"name":"Frozen Yogurt","calories":200,"fat":6,"carbs":24,"protein":4,"iron":"1%"},
    {"name":"Ice cream sandwich","calories":200,"fat":9,"carbs":37,"protein":4.3,"iron":"1%"},
    {"name":"Eclair","calories":300,"fat":16,"carbs":23,"protein":6,"iron":"7%"},
    {"name":"Cupcake","calories":300,"fat":3.7,"carbs":67,"protein":4.3,"iron":"8%"},
    {"name":"Gingerbread","calories":400,"fat":16,"carbs":49,"protein":3.9,"iron":"16%"},
    {"name":"Jelly bean","calories":400,"fat":0,"carbs":94,"protein":0,"iron":"0%"},
    {"name":"Lollipop","calories":400,"fat":0.2,"carbs":98,"protein":0,"iron":"2%"},
    {"name":"Honeycomb","calories":400,"fat":3.2,"carbs":87,"protein":6.5,"iron":"45%"},
    {"name":"Donut","calories":500,"fat":25,"carbs":51,"protein":4.9,"iron":"22%"},
    {"name":"KitKat","calories":500,"fat":26,"carbs":65,"protein":7,"iron":"6%"}
]
    """
)

frame = pd.DataFrame.from_dict(data)

# --------------------------------------------------------------------------------
# Configure table columns and options
# --------------------------------------------------------------------------------

# fmt: off
header_options = {
    "name":     {"text": "Dessert", "align": "start", "sortable": False},
    "calories": {"text": "Calories"},
    "fat":      {"text": "Fat (g)"},
    "carbs":    {"text": "Carbs (g)"},
    "protein":  {"text": "Protein (g)"},
    "iron":     {"text": "Iron (%)"},
}

# fmt: on
headers, rows = vuetify.dataframe_to_grid(frame, header_options)
table = {
    "headers": ("headers", headers),
    "items": ("rows", rows),
    "search": ("query", ""),
    "classes": "elevation-1 ma-4",
    "multi_sort": True,
    "dense": True,
    "items_per_page": 5,
}


# --------------------------------------------------------------------------------
# GUI
# --------------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.title.set_text("Vuetify table example")
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VTextField(
            v_model=("query", ""),
            placeholder="Search",
            dense=True,
            hide_details=True,
        )

    with layout.content:
        vuetify.VDataTable(**table)

if __name__ == "__main__":
    server.start()



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vtk trame-components trame-plotly
"""

import pandas as pd

# Plotly/chart imports
import plotly.express as px
import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.numpy_interface import dataset_adapter as dsa
from vtkmodules.vtkCommonCore import vtkIdTypeArray
from vtkmodules.vtkCommonDataModel import vtkDataObject, vtkSelection, vtkSelectionNode
from vtkmodules.vtkFiltersExtraction import vtkExtractSelection
from vtkmodules.vtkFiltersGeometry import vtkGeometryFilter
from vtkmodules.vtkInteractionStyle import (
    vtkInteractorStyleRubberBandPick,
)  # noqa

# VTK imports
from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridReader
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkDataSetMapper,
    vtkHardwareSelector,
    vtkRenderedAreaPicker,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

# Trame imports
from trame.app import get_server
from trame.assets.remote import HttpFile
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import plotly, trame, vuetify
from trame.widgets import vtk as vtk_widgets

# -----------------------------------------------------------------------------
# Data file information
# -----------------------------------------------------------------------------

dataset_file = HttpFile(
    "./data/disk_out_ref.vtu",
    "https://github.com/Kitware/trame/raw/master/examples/data/disk_out_ref.vtu",
    __file__,
)

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


# -----------------------------------------------------------------------------
# VTK
# -----------------------------------------------------------------------------

reader = vtkXMLUnstructuredGridReader()
reader.SetFileName(dataset_file.path)
reader.Update()
dataset = reader.GetOutput()

renderer = vtkRenderer()
renderer.SetBackground(1, 1, 1)
render_window = vtkRenderWindow()
render_window.AddRenderer(renderer)

rw_interactor = vtkRenderWindowInteractor()
rw_interactor.SetRenderWindow(render_window)
rw_interactor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

interactor_trackball = rw_interactor.GetInteractorStyle()
interactor_selection = vtkInteractorStyleRubberBandPick()
area_picker = vtkRenderedAreaPicker()
rw_interactor.SetPicker(area_picker)

surface_filter = vtkGeometryFilter()
surface_filter.SetInputConnection(reader.GetOutputPort())
surface_filter.SetPassThroughPointIds(True)

mapper = vtkDataSetMapper()
mapper.SetInputConnection(surface_filter.GetOutputPort())
actor = vtkActor()
actor.GetProperty().SetOpacity(0.5)
actor.SetMapper(mapper)

# Selection
selection_extract = vtkExtractSelection()
selection_mapper = vtkDataSetMapper()
selection_mapper.SetInputConnection(selection_extract.GetOutputPort())
selection_actor = vtkActor()
selection_actor.GetProperty().SetColor(1, 0, 1)
selection_actor.GetProperty().SetPointSize(5)
selection_actor.SetMapper(selection_mapper)
selection_actor.SetVisibility(0)

renderer.AddActor(actor)
renderer.AddActor(selection_actor)
renderer.ResetCamera()

selector = vtkHardwareSelector()
selector.SetRenderer(renderer)
selector.SetFieldAssociation(vtkDataObject.FIELD_ASSOCIATION_POINTS)

# vtkDataSet to DataFrame
py_ds = dsa.WrapDataObject(dataset)
pt_data = py_ds.PointData
cols = {}
for name in pt_data.keys():
    array = pt_data[name]
    shp = array.shape
    if len(shp) == 1:
        cols[name] = array
    else:
        for i in range(shp[1]):
            cols[name + "_%d" % i] = array[:, i]
DATAFRAME = pd.DataFrame(cols)
FIELD_NAMES = list(cols.keys())
SELECTED_IDX = []

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("figure_size", "scatter_x", "scatter_y")
def update_figure(figure_size, scatter_x, scatter_y, **kwargs):
    if figure_size is None:
        return

    # Generate figure
    bounds = figure_size.get("size", {})
    fig = px.scatter(
        DATAFRAME,
        x=scatter_x,
        y=scatter_y,
        width=bounds.get("width", 200),
        height=bounds.get("height", 200),
    )

    # Update selection settings
    fig.data[0].update(
        selectedpoints=SELECTED_IDX,
        selected={"marker": {"color": "red"}},
        unselected={"marker": {"opacity": 0.5}},
    )

    # Update chart
    ctrl.update_figure(fig)


# -----------------------------------------------------------------------------


@state.change("vtk_selection")
def update_interactor(vtk_selection, **kwargs):
    if vtk_selection:
        # remote view
        rw_interactor.SetInteractorStyle(interactor_selection)
        interactor_selection.StartSelect()
        # local view
        state.interactorSettings = VIEW_SELECT
    else:
        # remote view
        rw_interactor.SetInteractorStyle(interactor_trackball)
        # local view
        state.interactorSettings = VIEW_INTERACT


# -----------------------------------------------------------------------------


def on_chart_selection(selected_point_idxs):
    global SELECTED_IDX
    SELECTED_IDX = selected_point_idxs if selected_point_idxs else []
    npts = len(SELECTED_IDX)

    ids = vtkIdTypeArray()
    ids.SetNumberOfTuples(npts)
    for idx, p_id in enumerate(SELECTED_IDX):
        ids.SetTuple1(idx, p_id)
        idx += 1

    sel_node = vtkSelectionNode()
    sel_node.GetProperties().Set(
        vtkSelectionNode.CONTENT_TYPE(), vtkSelectionNode.INDICES
    )
    sel_node.GetProperties().Set(vtkSelectionNode.FIELD_TYPE(), vtkSelectionNode.POINT)
    sel_node.SetSelectionList(ids)
    sel = vtkSelection()
    sel.AddNode(sel_node)

    selection_extract.SetInputDataObject(0, py_ds.VTKObject)
    selection_extract.SetInputDataObject(1, sel)
    selection_extract.Update()
    selection_actor.SetVisibility(1)

    # Update 3D view
    ctrl.view_update()


def on_box_selection_change(selection):
    global SELECTED_IDX
    if selection.get("mode") == "remote":
        actor.GetProperty().SetOpacity(1)
        selector.SetArea(
            int(renderer.GetPickX1()),
            int(renderer.GetPickY1()),
            int(renderer.GetPickX2()),
            int(renderer.GetPickY2()),
        )
    elif selection.get("mode") == "local":
        camera = renderer.GetActiveCamera()
        camera_props = selection.get("camera")

        # Sync client view to server one
        camera.SetPosition(camera_props.get("position"))
        camera.SetFocalPoint(camera_props.get("focalPoint"))
        camera.SetViewUp(camera_props.get("viewUp"))
        camera.SetParallelProjection(camera_props.get("parallelProjection"))
        camera.SetParallelScale(camera_props.get("parallelScale"))
        camera.SetViewAngle(camera_props.get("viewAngle"))
        render_window.SetSize(selection.get("size"))

        actor.GetProperty().SetOpacity(1)
        render_window.Render()

        area = selection.get("selection")
        selector.SetArea(
            int(area[0]),
            int(area[2]),
            int(area[1]),
            int(area[3]),
        )

    # Common server selection
    s = selector.Select()
    n = s.GetNode(0)
    ids = dsa.vtkDataArrayToVTKArray(n.GetSelectionData().GetArray("SelectedIds"))
    surface = dsa.WrapDataObject(surface_filter.GetOutput())
    SELECTED_IDX = surface.PointData["vtkOriginalPointIds"][ids].tolist()

    selection_extract.SetInputConnection(surface_filter.GetOutputPort())
    selection_extract.SetInputDataObject(1, s)
    selection_extract.Update()
    selection_actor.SetVisibility(1)
    actor.GetProperty().SetOpacity(0.5)

    # Update scatter plot with selection
    update_figure(**state.to_dict())

    # Update 3D view
    ctrl.view_update()

    # disable selection mode
    state.vtk_selection = False


# -----------------------------------------------------------------------------
# Settings
# -----------------------------------------------------------------------------

DROPDOWN_STYLES = {
    "dense": True,
    "hide_details": True,
    "classes": "px-2",
    "style": "max-width: calc(25vw - 10px);",
}

CHART_STYLE = {
    "style": "position: absolute; left: 50%; transform: translateX(-50%);",
    "display_mode_bar": ("true",),
    "mode_bar_buttons_to_remove": (
        "chart_buttons",
        [
            "toImage",
            "resetScale2d",
            "zoomIn2d",
            "zoomOut2d",
            "toggleSpikelines",
            "hoverClosestCartesian",
            "hoverCompareCartesian",
        ],
    ),
    "display_logo": ("false",),
}

VTK_VIEW_SETTINGS = {
    "interactive_ratio": 1,
    "interactive_quality": 80,
}

VIEW_INTERACT = [
    {"button": 1, "action": "Rotate"},
    {"button": 2, "action": "Pan"},
    {"button": 3, "action": "Zoom", "scrollEnabled": True},
    {"button": 1, "action": "Pan", "alt": True},
    {"button": 1, "action": "Zoom", "control": True},
    {"button": 1, "action": "Pan", "shift": True},
    {"button": 1, "action": "Roll", "alt": True, "shift": True},
]

VIEW_SELECT = [{"button": 1, "action": "Select"}]


# -----------------------------------------------------------------------------
# UI
# -----------------------------------------------------------------------------

state.trame__title = "VTK selection"
ctrl.on_server_ready.add(ctrl.view_update)

with SinglePageLayout(server) as layout:
    layout.title.set_text("VTK & plotly")
    layout.icon.click = ctrl.view_reset_camera

    with layout.toolbar as tb:
        tb.dense = True
        vuetify.VSpacer()
        vuetify.VSelect(
            v_model=("scatter_y", FIELD_NAMES[1]),
            items=("fields", FIELD_NAMES),
            **DROPDOWN_STYLES,
        )
        vuetify.VSelect(
            v_model=("scatter_x", FIELD_NAMES[0]),
            items=("fields", FIELD_NAMES),
            **DROPDOWN_STYLES,
        )

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="fill-height pa-0 ma-0"):
            with vuetify.VRow(dense=True, style="height: 100%;"):
                with vuetify.VCol(
                    classes="pa-0",
                    style="border-right: 1px solid #ccc; position: relative;",
                ):
                    view = vtk_widgets.VtkRemoteView(
                        # view = vtk_widgets.VtkLocalView(
                        render_window,
                        box_selection=("vtk_selection",),
                        box_selection_change=(on_box_selection_change, "[$event]"),
                        # For VtkRemoteView
                        **VTK_VIEW_SETTINGS,
                        # For VtkLocalView
                        interactor_settings=("interactorSettings", VIEW_SELECT),
                    )
                    ctrl.view_update = view.update
                    ctrl.view_reset_camera = view.reset_camera
                    vuetify.VCheckbox(
                        small=True,
                        on_icon="mdi-selection-drag",
                        off_icon="mdi-rotate-3d",
                        v_model=("vtk_selection", False),
                        style="position: absolute; top: 0; right: 0; z-index: 1;",
                        dense=True,
                        hide_details=True,
                    )
                with vuetify.VCol(classes="pa-0"):
                    with trame.SizeObserver("figure_size"):
                        html_plot = plotly.Figure(
                            selected=(
                                on_chart_selection,
                                "[$event?.points.map(({pointIndex}) => pointIndex)]",
                            ),
                            **CHART_STYLE,
                        )
                        ctrl.update_figure = html_plot.update

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from tkinter import Tk, filedialog

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server()
state, ctrl = server.state, server.controller

# Keep track of the currently selected directory
state.selected_dir = None

root = Tk()

# Ensure the tkinter main window is hidden
root.withdraw()

# Ensure that the file browser will appear in front on Windows
root.wm_attributes("-topmost", 1)


@ctrl.set("open_directory")
def open_directory():
    kwargs = {
        "title": "Select Directory",
    }
    state.selected_dir = filedialog.askdirectory(**kwargs)


# -----------------------------------------------------------------------------
# UI setup
# -----------------------------------------------------------------------------

with DivLayout(server):
    html.Button("Select Directory", click=ctrl.open_directory)
    html.Div("{{ selected_dir }}")

# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/SimpleCone/ClientView.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/674f72774228bbcab5689417c1c5642230b1eab8

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

from vtkmodules.vtkFiltersSources import vtkConeSource

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "VTK Local rendering"

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

DEFAULT_RESOLUTION = 6

cone_generator = vtkConeSource()

# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("resolution")
def update_cone(resolution=DEFAULT_RESOLUTION, **kwargs):
    cone_generator.SetResolution(resolution)
    ctrl.mesh_update()


def update_reset_resolution():
    state.resolution = DEFAULT_RESOLUTION


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("Cone Application")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        vuetify.VDivider(vertical=True, classes="mx-2")

        with vuetify.VBtn(icon=True, click=update_reset_resolution):
            vuetify.VIcon("mdi-undo-variant")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            with vtk_widgets.VtkView() as view:
                ctrl.view_reset_camera = view.reset_camera
                with vtk_widgets.VtkGeometryRepresentation():
                    html_polydata = vtk_widgets.VtkPolyData(
                        "cone", dataset=cone_generator
                    )
                    ctrl.mesh_update = html_polydata.update


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



#!/usr/bin/env python

import json
import os
import subprocess

CLIENT_TYPE = os.environ.get("TRAME_CLIENT_TYPE", "vue3")


def run(apps_path, out_path):
    # Generate www content
    cmd = [
        "python",
        "-m",
        "trame.tools.www",
        "--output",
        out_path,
        "--client-type",
        CLIENT_TYPE,
    ]
    subprocess.run(cmd)

    # Generate app files index.html => {app_name}.html
    with open(apps_path, "r") as rf:
        apps_dict = json.load(rf)  # noqa

        for app_name, config in apps_dict.items():
            # handle custom modules for www
            web_modules = config.get("www_modules")
            client_type = config.get("client_type", CLIENT_TYPE)
            if web_modules is not None:
                cmd = [
                    "python",
                    "-m",
                    "trame.tools.www",
                    "--output",
                    out_path,
                    "--client-type",
                    client_type,
                    *web_modules,
                ]
                subprocess.run(cmd)

            # Create app.html file from index.html
            cmd = [
                "python",
                "-m",
                "trame.tools.app",
                "--input",
                out_path,
                "--name",
                app_name,
            ]
            subprocess.run(cmd)


if __name__ == "__main__":
    apps_path = "/opt/trame/apps.json"
    out_path = "/deploy/server/www"
    run(apps_path, out_path)



r"""
Installation requirements:
    pip install trame trame-vuetify trame-vega
"""

# -----------------------------------------------------------------------------
# More examples available at https://altair-viz.github.io/gallery/
# -----------------------------------------------------------------------------

import altair as alt
from vega_datasets import data

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vega, vuetify

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# Chart examples
# -----------------------------------------------------------------------------


def ScatterMatrix():
    """https://altair-viz.github.io/gallery/scatter_matrix.html"""
    source = data.cars()

    chart = (
        alt.Chart(source)
        .mark_circle()
        .encode(
            alt.X(alt.repeat("column"), type="quantitative"),
            alt.Y(alt.repeat("row"), type="quantitative"),
            color="Origin:N",
        )
        .properties(width=200, height=200)
        .repeat(
            row=["Horsepower", "Acceleration", "Miles_per_Gallon"],
            column=["Miles_per_Gallon", "Acceleration", "Horsepower"],
        )
        .interactive()
    )

    # Push chart to client
    ctrl.vega_update(chart)


# -----------------------------------------------------------------------------


def USIncomeByState():
    """https://altair-viz.github.io/gallery/us_incomebrackets_by_state_facet.html"""
    states = alt.topo_feature(data.us_10m.url, "states")
    source = data.income.url

    chart = (
        alt.Chart(source)
        .mark_geoshape()
        .encode(
            shape="geo:G",
            color="pct:Q",
            tooltip=["name:N", "pct:Q"],
            facet=alt.Facet("group:N", columns=3),
        )
        .transform_lookup(
            lookup="id", from_=alt.LookupData(data=states, key="id"), as_="geo"
        )
        .properties(
            width=300,
            height=175,
        )
        .project(type="albersUsa")
    )

    # Push chart to client
    ctrl.vega_update(chart)


# -----------------------------------------------------------------------------


def StackedDensityEstimates():
    """https://altair-viz.github.io/gallery/density_stack.html"""
    source = data.iris()

    chart = (
        alt.Chart(source)
        .transform_fold(
            ["petalWidth", "petalLength", "sepalWidth", "sepalLength"],
            as_=["Measurement_type", "value"],
        )
        .transform_density(
            density="value",
            bandwidth=0.3,
            groupby=["Measurement_type"],
            extent=[0, 8],
            counts=True,
            steps=200,
        )
        .mark_area()
        .encode(
            alt.X("value:Q"),
            alt.Y("density:Q", stack="zero"),
            alt.Color("Measurement_type:N"),
        )
        .properties(width=400, height=100)
    )

    ctrl.vega_update(chart)


def StreamGraph():
    """https://altair-viz.github.io/gallery/streamgraph.html"""
    source = data.unemployment_across_industries.url

    chart = (
        alt.Chart(source)
        .mark_area()
        .encode(
            alt.X(
                "yearmonth(date):T",
                axis=alt.Axis(format="%Y", domain=False, tickSize=0),
            ),
            alt.Y("sum(count):Q", stack="center", axis=None),
            alt.Color("series:N", scale=alt.Scale(scheme="category20b")),
        )
        .properties(width="container")
        .interactive()
    )

    ctrl.vega_update(chart)


# -----------------------------------------------------------------------------
# Callbacks
# -----------------------------------------------------------------------------


@state.change("active")
def update_chart(active="ScatterMatrix", **kwargs):
    globals()[active]()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

example_charts = [
    {"text": "Scatter Matrix", "value": "ScatterMatrix"},
    {"text": "US Income By State", "value": "USIncomeByState"},
    # {"text": "Stacked Density Estimates", "value": "StackedDensityEstimates"},
    {"text": "StreamGraph", "value": "StreamGraph"},
]

state.trame__title = "Altair charts"

with SinglePageLayout(server) as layout:
    layout.title.set_text("trame ❤️ altair charts")

    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSelect(
            v_model=("active", "ScatterMatrix"),
            items=("examples", example_charts),
            dense=True,
            hide_details=True,
        )

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="text-center"):
            fig = vega.Figure(style="width: 100%;")
            ctrl.vega_update = fig.update

# -----------------------------------------------------------------------------
# Start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



r"""
From the directory containing the static content for a trame application to work,
generate another application specific HTML file.
"""

import argparse
import re
import sys
from pathlib import Path

RETRY_PATTERN = re.compile(r'data-launcher-retry=".+"')


def update_retry_file(input_file, output_file, retry_array):
    # Read in the file
    with open(input_file, "r") as f_in:
        content = f_in.read()
        patched_content = RETRY_PATTERN.sub(
            f'data-launcher-retry="{retry_array}"', content
        )
        with open(output_file, "w") as f_out:
            f_out.write(patched_content)


def main():
    parser = argparse.ArgumentParser(
        description="HTML app file generator for trame applications"
    )

    parser.add_argument(
        "--retry",
        default="[]",
        help="Retry array for launcher. Times are in ms.",
    )

    parser.add_argument(
        "--input",
        help="Input file to use as template",
    )

    args, _ = parser.parse_known_args()

    # Handle input
    input_file = Path(args.input)
    if not input_file.exists():
        parser.print_help()
        sys.exit(0)

    if input_file.is_dir():
        input_file = input_file / "index.html"

    if not input_file.exists():
        parser.print_help()
        sys.exit(0)

    update_retry_file(input_file, input_file, args.retry)


if __name__ == "__main__":
    main()



import time

import vtkmodules.vtkRenderingOpenGL2  # noqa
from vtkmodules.vtkFiltersGeometry import vtkGeometryFilter
from vtkmodules.vtkImagingCore import vtkRTAnalyticSource

# VTK factory initialization
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkDataSetMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk, vuetify

# -----------------------------------------------------------------------------
# Trame initialization
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

state.trame__title = "Geometry export"

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

DEFAULT_RESOLUTION = 10

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

source = vtkRTAnalyticSource()
filter = vtkGeometryFilter()
filter.SetInputConnection(source.GetOutputPort())
mapper = vtkDataSetMapper()
actor = vtkActor()
mapper.SetInputConnection(filter.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

filter.Update()
_min, _max = filter.GetOutput().GetPointData().GetScalars().GetRange()
mapper.SetScalarRange(_min, _max)
actor.GetProperty().SetEdgeVisibility(1)
actor.GetProperty().SetEdgeColor(1, 1, 1)

# -----------------------------------------------------------------------------


def update_view():
    t0 = time.time()
    ctrl.view_update()
    t1 = time.time()
    print(f"Server: Updated scene in {t1 - t0:.3f}s")


@state.change("resolution")
def update_resolution(resolution=DEFAULT_RESOLUTION, **kwargs):
    source.SetWholeExtent(
        -resolution, resolution, -resolution, resolution, -resolution, resolution
    )
    ctrl.view_reset_camera()
    update_view()


# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("Geometry export")

    with layout.toolbar as tb:
        vuetify.VSpacer()
        tb.add_child("{{ resolution }}")
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=10,
            max=100,
            step=1,
            hide_details=True,
            dense=True,
            style="max-width: 300px",
        )
        vuetify.VBtn("Update", click=update_view)

    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
        ):
            # view = vtk.VtkLocalView(renderWindow)
            # view = vtk.VtkRemoteView(renderWindow)
            view = vtk.VtkRemoteLocalView(renderWindow, mode="local")
            ctrl.view_update = view.update
            ctrl.view_reset_camera = view.reset_camera


ctrl.on_server_ready.add(ctrl.view_update)

# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



from pathlib import Path

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import helper
from trame.widgets import vuetify3 as vuetify

server = get_server(client_type="vue3")
state = server.state
controller = server.controller


SCRIPT_PATH = Path(__file__).with_name("707.js")
draggable_module = dict(
    serve={"__trame_vuedraggable": str(SCRIPT_PATH.parent.absolute())},
    scripts=[
        "https://cdn.jsdelivr.net/npm/sortablejs@1.10.2/Sortable.min.js",
        "https://cdn.jsdelivr.net/npm/vuedraggable@next/dist/vuedraggable.umd.min.js",
        "__trame_vuedraggable/707.js",
    ],
    vue_use=["trame_vuedraggable"],
)


# helper class
Draggable = helper.create_class(
    "Draggable", "draggable", module=draggable_module, properties=["item_key"]
)

state.tags = [
    {"id": "0", "name": "Shopping"},
    {"id": "1", "name": "Art"},
    {"id": "2", "name": "Tech"},
    {"id": "3", "name": "Creative Writing"},
]

# layout
with DivLayout(server) as layout:
    with vuetify.VApp():
        with vuetify.VContainer(classes="pa-4"):
            with vuetify.VCard(max_width=400, classes="mx-auto"):
                with Draggable(v_model=("tags",), item_key="id") as drag:
                    with vuetify.Template(
                        v_slot_item="{element}",
                        __properties=[("v_slot_items", "v-slot:items")],
                    ):
                        with vuetify.VCard(key="element.id"):
                            vuetify.VCardText("{{ element.name }}")

server.start()



import vtkmodules.vtkRenderingOpenGL2  # noqa

# -----------------------------------------------------------------------------
# VTK
# -----------------------------------------------------------------------------
from vtkmodules.vtkFiltersSources import vtkConeSource
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleSwitch  # noqa
from vtkmodules.vtkIOImage import vtkPNGWriter
from vtkmodules.vtkRenderingCore import (
    vtkActor,
    vtkPolyDataMapper,
    vtkRenderer,
    vtkRenderWindow,
    vtkRenderWindowInteractor,
    vtkWindowToImageFilter,
)

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import vtk, vuetify

DEFAULT_RESOLUTION = 6

renderer = vtkRenderer()
renderWindow = vtkRenderWindow()
renderWindow.AddRenderer(renderer)

renderWindowInteractor = vtkRenderWindowInteractor()
renderWindowInteractor.SetRenderWindow(renderWindow)
renderWindowInteractor.GetInteractorStyle().SetCurrentStyleToTrackballCamera()

cone_source = vtkConeSource()
mapper = vtkPolyDataMapper()
actor = vtkActor()
mapper.SetInputConnection(cone_source.GetOutputPort())
actor.SetMapper(mapper)
renderer.AddActor(actor)
renderer.ResetCamera()
renderWindow.Render()

# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller


@state.change("resolution")
def update_resolution(resolution, **kwargs):
    cone_source.SetResolution(int(resolution))
    ctrl.view_update()


@ctrl.trigger("download_screenshot")
def download_screenshot():
    w2if = vtkWindowToImageFilter()
    w2if.SetInput(renderWindow)
    w2if.SetInputBufferTypeToRGB()
    w2if.ReadFrontBufferOff()
    #
    writer1 = vtkPNGWriter()
    writer1.SetInputConnection(w2if.GetOutputPort())
    writer1.SetWriteToMemory(True)
    writer1.Write()
    #
    return server.protocol.addAttachment(memoryview(writer1.GetResult()))


with SinglePageLayout(server) as layout:
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VSlider(
            v_model=("resolution", DEFAULT_RESOLUTION),
            min=3,
            max=60,
            style="max-width: 300px",
            dense=True,
            hide_details=True,
        )
        with vuetify.VBtn(
            icon=True,
            click="utils.download('vtk.png', trigger('download_screenshot'), 'image/png')",
        ):
            vuetify.VIcon("mdi-camera")

    with layout.content:
        with vuetify.VContainer(classes="pa-0 fill-height", fluid=True):
            view = vtk.VtkRemoteView(renderWindow)
            ctrl.view_update = view.update


# -----------------------------------------------------------------------------
# start server
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import trame.widgets.html as h
import trame.widgets.vuetify3 as v3
from trame.app import get_server
from trame.ui.vuetify3 import SinglePageLayout, VAppLayout


class Example2:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue3")
        self.layout = self._build_ui()
        self.count = 0

    def _build_ui(self):
        with VAppLayout(self.server, template_name="example2") as layout:
            with layout:
                v3.VBtn(
                    "test (spoiler, I work)",
                    style="background-color:gray;",
                    click=self.test,
                )
                with v3.VContainer() as self.div:
                    h.Div("YES")
        return layout

    def test(self):
        self.count += 1

        self.div.clear()
        with self.div:
            h.Div(f"test {self.count}")
        self.layout.flush_content()


class Example3:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue3")
        self.layout = self._build_ui()
        self.count = 0

    def _build_ui(self):
        with VAppLayout(self.server, template_name="example_3") as layout:
            with layout:
                v3.VBtn(
                    "test (spoiler, I don't work)",
                    style="background-color:gray;",
                    click=self.test,
                )
                with v3.VContainer() as self.div:
                    h.Div("YES")
        return layout

    def test(self):
        self.count += 1

        self.div.clear()
        with self.div:
            h.Div(f"test {self.count}")
        self.layout.flush_content()


class Example:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue3")
        self.layout = self._build_ui()
        self.example2 = Example2()
        self.example3 = Example3()

    def _build_ui(self):
        with SinglePageLayout(self.server) as layout:
            with layout.toolbar:
                v3.VBtn(
                    "Not Working",
                    click="window.open('/?ui=example_3', target='_blank')",
                )
                v3.VBtn(
                    "Working", click="window.open('/?ui=example2', target='_blank')"
                )


if __name__ == "__main__":
    app = Example()
    app.server.start()



r"""
https://vuetifyjs.com/en/components/tooltips/

    <v-tooltip text="Tooltip">
      <template v-slot:activator="{ props }">
        <v-btn v-bind="props">Hover over me</v-btn>
      </template>
    </v-tooltip>
"""

from trame.app import get_server
from trame.ui.vuetify3 import VAppLayout
from trame.widgets import vuetify3

server = get_server(client_type="vue3")

with VAppLayout(server):
    with vuetify3.VTooltip(text="Tooltip"):
        with vuetify3.Template(v_slot_activator="{ props }"):
            vuetify3.VBtn("Hover over me", v_bind="props")

server.start(port=10003)



import vtk

from trame.app import get_server
from trame.decorators import TrameApp, change
from trame.ui.vuetify3 import VAppLayout
from trame.widgets import html
from trame.widgets import vtk as vtk_widgets
from trame.widgets import vuetify3 as v3

# -----------------------------------------------------------------------------
# VTK pipeline definition
# -----------------------------------------------------------------------------


def setup_vtk():
    renderer = vtk.vtkRenderer()
    render_window = vtk.vtkRenderWindow(off_screen_rendering=True)
    render_window.AddRenderer(renderer)

    render_window_interactor = vtk.vtkRenderWindowInteractor(
        render_window=render_window
    )
    render_window_interactor.interactor_style.SetCurrentStyleToTrackballCamera()

    # Sources
    sphere_source = vtk.vtkSphereSource(
        theta_resolution=16,
        phi_resolution=16,
    )
    cone_source = vtk.vtkConeSource(
        radius=0.1,
        height=0.2,
        resolution=30,
    )

    # Pipeline
    glyph_filter = vtk.vtkGlyph3D(
        source_connection=cone_source.output_port,
        orient=True,
        vector_mode=1,  # normals
    )

    pipeline = (
        sphere_source
        >> vtk.vtkPolyDataNormals(compute_cell_normals=1)
        >> vtk.vtkCellCenters()
        >> glyph_filter
    )

    # Rendering
    renderer.AddActor(
        vtk.vtkActor(
            mapper=vtk.vtkPolyDataMapper(
                input_connection=glyph_filter.output_port,
            ),
        )
    )
    renderer.ResetCamera()
    render_window.Render()

    return render_window, sphere_source, cone_source, pipeline


# -----------------------------------------------------------------------------
# GUI helpers
# -----------------------------------------------------------------------------


class TitleWithStatistic(v3.VCardTitle):
    def __init__(self, name, title, width):
        super().__init__(title, classes="d-flex align-center")

        with self:
            v3.VSpacer()
            with v3.VChip(
                size="small",
                variant="outlined",
                style=f"width: {width}rem;",
            ):
                v3.VIcon("mdi-dots-triangle", start=True)
                html.Span(f"{{{{ {name}_points.toLocaleString() }}}}")
            with v3.VChip(
                size="small",
                variant="outlined",
                classes="ml-2",
                style=f"width: {width}rem;",
                v_show=f"{name}_points !== {name}_cells",
            ):
                v3.VIcon("mdi-triangle-outline", start=True)
                html.Span(f"{{{{ {name}_cells.toLocaleString() }}}}")


def slider(title, name, default_value, min_value, max_value, step_value):
    v3.VLabel(f"{title}: {{{{ {name} }}}}")
    v3.VSlider(
        v_model=(name, default_value),
        min=min_value,
        step=step_value,
        max=max_value,
        hide_details=True,
    )


# -----------------------------------------------------------------------------
# Application
# -----------------------------------------------------------------------------


@TrameApp()
class Viewer:
    def __init__(self, server=None):
        self.rw, self.sphere, self.cone, self.pipeline = setup_vtk()
        self.server = get_server(server)
        self._build_ui()

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    @change("cone_resolution", "cone_height", "cone_radius")
    def update_cone(self, cone_resolution, cone_height, cone_radius, **_):
        self.cone.resolution = cone_resolution
        self.cone.height = cone_height
        self.cone.radius = cone_radius

        # Execute filter for output extraction
        cone_dataset = self.cone()
        output_dataset = self.pipeline()

        # Update UI with new statistics
        self.state.update(
            {
                "cone_points": cone_dataset.number_of_points,
                "cone_cells": cone_dataset.number_of_cells,
                "total_points": output_dataset.number_of_points,
                "total_cells": output_dataset.number_of_cells,
            }
        )

        self.ctrl.view_update()

    @change("sphere_resolution")
    def update_sphere(self, sphere_resolution, **_):
        self.sphere.theta_resolution = sphere_resolution
        self.sphere.phi_resolution = sphere_resolution

        # Execute filter for output extraction
        sphere_dataset = self.sphere()
        output_dataset = self.pipeline()

        # Update UI with new statistics
        self.state.update(
            {
                "sphere_points": sphere_dataset.number_of_points,
                "sphere_cells": sphere_dataset.number_of_cells,
                "total_points": output_dataset.number_of_points,
                "total_cells": output_dataset.number_of_cells,
            }
        )
        self.ctrl.view_update()

    def _build_ui(self):
        with VAppLayout(self.server, fill_height=True) as self.ui:
            with v3.VCard(
                style="z-index: 1;",
                classes="position-absolute w-33 top-0 left-0 ma-4",
            ):
                # Sphere
                TitleWithStatistic("sphere", "Sphere", 4)
                v3.VDivider()
                with v3.VCardText():
                    slider("Resolution", "sphere_resolution", 16, 8, 32, 1)

                # Cone
                v3.VDivider()
                TitleWithStatistic("cone", "Cone", 3)
                v3.VDivider()
                with v3.VCardText():
                    slider("Resolution", "cone_resolution", 30, 3, 24, 1)
                    slider("Height", "cone_height", 0.2, 0.01, 0.5, 0.01)
                    slider("Radius", "cone_radius", 0.1, 0.01, 0.2, 0.01)

                # Result
                v3.VDivider()
                TitleWithStatistic("total", "Result", 5)

            with vtk_widgets.VtkRemoteView(self.rw, interactive_ratio=1) as view:
                self.ctrl.view_update = view.update
                self.ctrl.view_reset_camera = view.reset_camera


def main():
    app = Viewer()
    app.server.start()


if __name__ == "__main__":
    main()



r"""
Installation requirements:
    pip install trame
"""

from trame.app import get_server
from trame.ui.html import DivLayout

server = get_server()

html = """
<h3>Welcome to trame...</h3>

<div>
    <i>Hello</i> <b>World</b>
</div>
"""

with DivLayout(server) as layout:
    layout.root.add_child(html)

if __name__ == "__main__":
    server.start()



from trame.app import get_server
from trame.app.file_upload import ClientFile
from trame.ui.html import DivLayout
from trame.widgets import html

server = get_server()


def upload(files):
    for file in files:
        file_helper = ClientFile(file)
        print(file_helper.info)


with DivLayout(server) as a:
    html.Input(
        type="file",
        multiple=True,
        change=(upload, "[$event.target.files]"),
        __events=["change"],
    )


if __name__ == "__main__":
    server.start()



r"""
Version for trame 1.x - https://github.com/Kitware/trame/blob/release-v1/examples/VTK/Applications/FiniteElementAnalysis/app_client_view.py
Delta v1..v2          - https://github.com/Kitware/trame/commit/03f28bb0084490acabf218264b96a1dbb3a17f19

Installation requirements:
    pip install trame trame-vuetify trame-vtk
"""

import io
import os

import numpy as np
import pandas as pd
from vtkmodules.numpy_interface.dataset_adapter import numpyTovtkDataArray as np2da
from vtkmodules.util import vtkConstants
from vtkmodules.vtkCommonCore import vtkIdList, vtkPoints
from vtkmodules.vtkCommonDataModel import vtkCellArray, vtkUnstructuredGrid
from vtkmodules.vtkFiltersCore import vtkThreshold

from trame.app import get_server
from trame.ui.vuetify import SinglePageLayout
from trame.widgets import trame, vuetify
from trame.widgets import vtk as vtk_widgets

# -----------------------------------------------------------------------------
# Constants
# -----------------------------------------------------------------------------

VIEW_INTERACT = [
    {"button": 1, "action": "Rotate"},
    {"button": 2, "action": "Pan"},
    {"button": 3, "action": "Zoom", "scrollEnabled": True},
    {"button": 1, "action": "Pan", "alt": True},
    {"button": 1, "action": "Zoom", "control": True},
    {"button": 1, "action": "Pan", "shift": True},
    {"button": 1, "action": "Roll", "alt": True, "shift": True},
]

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server(client_type="vue2")
state, ctrl = server.state, server.controller

# -----------------------------------------------------------------------------
# VTK pipeline
# -----------------------------------------------------------------------------

vtk_idlist = vtkIdList()
vtk_grid = vtkUnstructuredGrid()
vtk_filter = vtkThreshold()
vtk_filter.SetInputData(vtk_grid)
field_to_keep = "my_array"


@state.change("nodes_file", "elems_file", "field_file")
def update_grid(nodes_file, elems_file, field_file, **kwargs):
    state.picking_modes = []
    if not nodes_file:
        return

    if not elems_file:
        return

    nodes_bytes = nodes_file.get("content")
    elems_bytes = elems_file.get("content")

    if isinstance(nodes_bytes, list):
        nodes_bytes = b"".join(nodes_bytes)

    if isinstance(elems_bytes, list):
        elems_bytes = b"".join(elems_bytes)

    df_nodes = pd.read_csv(
        io.StringIO(nodes_bytes.decode("utf-8")),
        delim_whitespace=True,
        header=None,
        skiprows=1,
        names=["id", "x", "y", "z"],
    )

    df_nodes["id"] = df_nodes["id"].astype(int)
    df_nodes = df_nodes.set_index("id", drop=True)
    # fill missing ids in range as VTK uses position (index) to map cells to points
    df_nodes = df_nodes.reindex(
        np.arange(df_nodes.index.min(), df_nodes.index.max() + 1), fill_value=0
    )

    df_elems = pd.read_csv(
        io.StringIO(elems_bytes.decode("utf-8")),
        skiprows=1,
        header=None,
        delim_whitespace=True,
        engine="python",
        index_col=None,
    ).sort_values(0)
    # order: 0: eid, 1: eshape, 2+: nodes, iloc[:,0] is index
    df_elems.iloc[:, 0] = df_elems.iloc[:, 0].astype(int)

    n_nodes = df_elems.iloc[:, 1].map(
        lambda x: int("".join(i for i in x if i.isdigit()))
    )
    df_elems.insert(2, "n_nodes", n_nodes)
    # fill missing ids in range as VTK uses position (index) to map data to cells
    new_range = np.arange(df_elems.iloc[:, 0].min(), df_elems.iloc[:, 0].max() + 1)
    df_elems = df_elems.set_index(0, drop=False).reindex(new_range, fill_value=0)

    # mapping specific to Ansys Mechanical data
    vtk_shape_id_map = {
        "Tet4": vtkConstants.VTK_TETRA,
        "Tet10": vtkConstants.VTK_QUADRATIC_TETRA,
        "Hex8": vtkConstants.VTK_HEXAHEDRON,
        "Hex20": vtkConstants.VTK_QUADRATIC_HEXAHEDRON,
        "Tri6": vtkConstants.VTK_QUADRATIC_TRIANGLE,
        "Quad8": vtkConstants.VTK_QUADRATIC_QUAD,
        "Tri3": vtkConstants.VTK_TRIANGLE,
        "Quad4": vtkConstants.VTK_QUAD,
        "Wed15": vtkConstants.VTK_QUADRATIC_WEDGE,
    }
    df_elems["cell_types"] = np.nan
    df_elems.loc[df_elems.loc[:, 0] > 0, "cell_types"] = df_elems.loc[
        df_elems.loc[:, 0] > 0, 1
    ].map(
        lambda x: (
            vtk_shape_id_map[x.strip()]
            if x.strip() in vtk_shape_id_map.keys()
            else np.nan
        )
    )
    df_elems = df_elems.dropna(subset=["cell_types"], axis=0)

    # convert dataframes to vtk-desired format
    points = df_nodes[["x", "y", "z"]].to_numpy()
    cell_types = df_elems["cell_types"].to_numpy()
    n_nodes = df_elems.loc[:, "n_nodes"].to_numpy()
    # subtract starting node id from all grid references in cells to avoid
    # filling from 0 to first used node (in case mesh doesn't start at 1)
    p = df_elems.iloc[:, 3:-1].to_numpy() - df_nodes.index.min()
    # if you need to, re-order nodes here-ish
    a = np.hstack((n_nodes.reshape((len(n_nodes), 1)), p))
    # convert to flat numpy array
    cells = a.ravel()
    # remove nans (due to elements with different no. of nodes)
    cells = cells[np.logical_not(np.isnan(cells))]
    cells = cells.astype(int)

    # update grid
    vtk_pts = vtkPoints()
    vtk_pts.SetData(np2da(points))
    vtk_grid.SetPoints(vtk_pts)

    vtk_cells = vtkCellArray()
    vtk_cells.SetCells(
        cell_types.shape[0], np2da(cells, array_type=vtkConstants.VTK_ID_TYPE)
    )
    vtk_grid.SetCells(
        np2da(cell_types, array_type=vtkConstants.VTK_UNSIGNED_CHAR), vtk_cells
    )

    # Add field if any
    if field_file:
        field_bytes = field_file.get("content")
        if isinstance(field_bytes, list):
            field_bytes = b"".join(field_bytes)
        df_elem_data = pd.read_csv(
            io.StringIO(field_bytes.decode("utf-8")),
            delim_whitespace=True,
            header=None,
            skiprows=1,
            names=["id", "val"],
        )
        df_elem_data = df_elem_data.sort_values("id").set_index("id", drop=True)
        # fill missing ids in range as VTK uses position (index) to map data to cells
        df_elem_data = df_elem_data.reindex(
            np.arange(df_elems.index.min(), df_elems.index.max() + 1), fill_value=0.0
        )
        np_val = df_elem_data["val"].to_numpy()
        # assign data to grid with the name 'my_array'
        vtk_array = np2da(np_val, name=field_to_keep)
        vtk_grid.GetCellData().SetScalars(vtk_array)
        state.full_range = vtk_array.GetRange()
        state.threshold_range = list(vtk_array.GetRange())
        state.picking_modes = ["hover"]

    ctrl.mesh_update()


@state.change("threshold_range")
def update_filter(threshold_range, **kwargs):
    vtk_filter.SetLowerThreshold(threshold_range[0])
    vtk_filter.SetUpperThreshold(threshold_range[1])
    ctrl.threshold_update()


def reset():
    state.update(
        {
            "mesh": None,
            "threshold": None,
            "nodes_file": None,
            "elems_file": None,
            "field_file": None,
        }
    )


@state.change("pick_data")
def update_tooltip(pick_data, pixel_ratio, **kwargs):
    state.tooltip = ""
    state.tooltip_style = {"display": "none"}
    data = pick_data

    if data:
        xyx = data["worldPosition"]
        idx = vtk_grid.FindPoint(xyx)
        field = vtk_grid.GetCellData().GetArray(0)
        if idx > -1 and field:
            messages = []
            vtk_grid.GetPointCells(idx, vtk_idlist)
            for i in range(vtk_idlist.GetNumberOfIds()):
                cell_idx = vtk_idlist.GetId(i)
                value = field.GetValue(cell_idx)
                value_str = f"{value:.2f}"
                messages.append(f"Scalar: {value_str}")

            if len(messages):
                x, y, z = data["displayPosition"]
                state.tooltip = messages[0]
                state.tooltip_style = {
                    "position": "absolute",
                    "left": f"{(x / pixel_ratio) + 10}px",
                    "bottom": f"{(y / pixel_ratio) + 10}px",
                    "zIndex": 10,
                    "pointerEvents": "none",
                }


# -----------------------------------------------------------------------------
# Web App setup
# -----------------------------------------------------------------------------

file_style = {
    "dense": True,
    "hide_details": True,
    "style": "max-width: 200px",
    "class": "mx-2",
    "small_chips": True,
    "clearable": ("false",),
    "accept": ".txt",
}

state.trame__title = "FEA - Mesh viewer"

with SinglePageLayout(server) as layout:
    layout.title.set_text("Mesh Viewer")
    layout.icon.click = reset

    # Let the server know the browser pixel ratio
    trame.ClientTriggers(mounted="pixel_ratio = window.devicePixelRatio")

    # Toolbar ----------------------------------------
    with layout.toolbar:
        vuetify.VSpacer()
        vuetify.VRangeSlider(
            thumb_size=16,
            thumb_label=True,
            label="Threshold",
            v_if=("threshold",),
            v_model=("threshold_range", [0, 1]),
            min=("full_range[0]",),
            max=("full_range[1]",),
            dense=True,
            hide_details=True,
            style="max-width: 400px",
        )
        vuetify.VFileInput(
            v_show=("!mesh",),
            prepend_icon="mdi-vector-triangle",
            v_model=("nodes_file", None),
            placeholder="Nodes",
            **file_style,
        )
        vuetify.VFileInput(
            v_show=("!mesh",),
            prepend_icon="mdi-dots-triangle",
            v_model=("elems_file", None),
            placeholder="Elements",
            **file_style,
        )
        vuetify.VFileInput(
            v_show=("!threshold",),
            prepend_icon="mdi-gradient",
            v_model=("field_file", None),
            placeholder="Field",
            **file_style,
        )
        with vuetify.VBtn(v_if=("mesh",), icon=True, click=ctrl.view_reset_camera):
            vuetify.VIcon("mdi-crop-free")

        vuetify.VProgressLinear(
            indeterminate=True, absolute=True, bottom=True, active=("trame__busy",)
        )

        trame.ClientStateChange(value="mesh", change=ctrl.view_reset_camera)

    # Content ----------------------------------------
    with layout.content:
        with vuetify.VContainer(
            fluid=True,
            classes="pa-0 fill-height",
            style="position: relative",
        ):
            with vtk_widgets.VtkView(
                ref="view",
                background=("[0.8, 0.8, 0.8]",),
                hover="pick_data = $event",
                picking_modes=("picking_modes", []),
                interactor_settings=("interactor_settings", VIEW_INTERACT),
            ) as view:
                ctrl.view_update = view.update
                ctrl.view_reset_camera = view.reset_camera
                with vtk_widgets.VtkGeometryRepresentation(
                    v_if=("mesh",),
                    property=(
                        """{
                            representation: threshold ? 1 : 2,
                            color: threshold ? [0.3, 0.3, 0.3] : [1, 1, 1],
                            opacity: threshold ? 0.2 : 1
                            }""",
                    ),
                ):
                    mesh = vtk_widgets.VtkMesh("mesh", dataset=vtk_grid)
                    ctrl.mesh_update = mesh.update

                with vtk_widgets.VtkGeometryRepresentation(
                    v_if=("threshold",),
                    color_data_range=("full_range", [0, 1]),
                ):
                    threshold = vtk_widgets.VtkMesh(
                        "threshold", dataset=vtk_filter, field_to_keep=field_to_keep
                    )
                    ctrl.threshold_update = threshold.update
            with vuetify.VCard(
                style=("tooltip_style", {"display": "none"}),
                elevation=2,
                outlined=True,
            ):
                vuetify.VCardText("<pre>{{ tooltip }}</pre>")


# Variables not defined within HTML but used
state.update(
    {
        "pixel_ratio": 1,
        "pick_data": None,
        "tooltip": "",
    }
)

# -----------------------------------------------------------------------------
# Use --data to skip file upload
# -----------------------------------------------------------------------------

parser = server.cli
parser.add_argument("--data", help="Unstructured file path", dest="data")
args = parser.parse_args()
if args.data:
    from vtkmodules.vtkIOXML import vtkXMLUnstructuredGridReader

    reader = vtkXMLUnstructuredGridReader()
    reader.SetFileName(os.path.abspath(args.data))
    reader.Update()
    vtu = reader.GetOutput()
    vtk_grid.ShallowCopy(vtu)

    vtk_array = vtu.GetCellData().GetScalars()
    full_min, full_max = vtk_array.GetRange()
    state.full_range = [full_min, full_max]
    state.threshold_range = [full_min, full_max]
    state.picking_modes = ["hover"]
    ctrl.mesh_update()
    ctrl.threshold_update()

# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.start()



import asyncio

from trame.app import get_server
from trame.decorators import TrameApp, controller
from trame.ui.html import DivLayout
from trame.widgets import trame


def toggle_node(node, node_id):
    if node.get("id") == node_id:
        new_visible = (node.get("visible") + 1) % 2
        return {**node, "visible": new_visible}
    return node


@TrameApp()
class App:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue3")
        self.ui = self._build_ui()

    @property
    def state(self):
        return self.server.state

    def toggle_visibility(self, node_id):
        with self.state:
            self.state.pipeline = [toggle_node(n, node_id) for n in self.state.pipeline]

    @controller.add_task("on_server_ready")
    async def auto_toggle(self, **kwargs):
        while True:
            for node_id in ["1", "2"]:
                await asyncio.sleep(0.1)
                self.toggle_visibility(node_id)

    def _build_ui(self):
        with DivLayout(self.server):
            trame.GitTree(
                sources=(
                    "pipeline",
                    [
                        {"id": "1", "parent": "0", "visible": 1, "name": "Mesh"},
                        {"id": "2", "parent": "1", "visible": 1, "name": "Contour"},
                    ],
                ),
            )


if __name__ == "__main__":
    app = App()
    app.server.start()



import json

from trame.app import get_server
from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import vuetify

server = get_server(client_type="vue2")
state = server.state
ctrl = server.controller

state.sgnodeList = json.loads(
    """[
    {
        "name": "First",
        "visible": false,
        "selfcolor": "#ff643d",
        "selected": true
    },
    {
        "name": "Second",
        "visible": true,
        "selfcolor": "green",
        "selected": false
    }
]"""
)


def changeSelected():
    print("changing selected")


with SinglePageWithDrawerLayout(server, style="overflow: hidden") as layout:
    layout.title.set_text("My App")

    with layout.drawer:
        with vuetify.VList(shaped=True):
            with vuetify.VListItem(v_for="val, idx in sgnodeList", key="idx"):
                vuetify.VSwitch(
                    v_model=("val.visible",),
                    color=("val.selfcolor",),
                )
                with vuetify.VListItemContent():
                    vuetify.VListItemTitle("{{val.name}}")

                vuetify.VCheckbox(
                    v_model=("val.selected",),
                    click=changeSelected,
                )

if __name__ == "__main__":
    server.start()



# export PYTHONPATH=/Applications/ParaView-5.12.0-RC1.app/Contents/Python/
# export DYLD_LIBRARY_PATH=/Applications/ParaView-5.12.0-RC1.app/Contents/Libraries/

from paraview import simple

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html, paraview

# create a new 'IOSS Reader'
disk_out_refex2 = simple.IOSSReader(
    registrationName="disk_out_ref.ex2",
    FileName=[
        "/Applications/ParaView-5.12.0-RC1.app/Contents/examples/disk_out_ref.ex2"
    ],
)

# get active view
renderView1 = simple.GetActiveViewOrCreate("RenderView")

# show data in view
disk_out_refex2Display = simple.Show(
    disk_out_refex2, renderView1, "UnstructuredGridRepresentation"
)

# trace defaults for the display properties.
disk_out_refex2Display.Representation = "Surface"


# update the view to ensure updated data information
renderView1.Update()

# change representation type
disk_out_refex2Display.SetRepresentationType("Surface LIC")


server = get_server(client_type="vue3")
with DivLayout(server) as layout:
    layout.root.style = "width: 100vw; height: 100vh; margin: 0; padding: 0;"
    reset_camera = paraview.VtkRemoteView(renderView1).reset_camera
    html.Button(
        "Reset", style="position: absolute; top: 10px; left: 10px;", click=reset_camera
    )

server.start()



from pathlib import Path

from trame.app import get_server
from trame.ui.html import DivLayout
from trame.widgets import html

DIRECTORY = Path(__file__).parent.parent.parent / "docs/vitepress/assets/images/apps"

server = get_server()
server.enable_module({"serve": {"data": str(DIRECTORY.absolute())}})

with DivLayout(server):
    for file in DIRECTORY.iterdir():
        html.Img(src=f"/data/{file.name}", style="width: 200px;")


if __name__ == "__main__":
    server.start()
